#!/usr/bin/env python

""" monitor -- connection monitoring process

This process is spawned as a child of the daemon, and is responsible
for monitoring connection status and initiating autoreconnection
when appropriate.

"""
#
#   Copyright (C) 2007 - 2009 Adam Blackburn
#   Copyright (C) 2007 - 2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gobject
import time

from dbus import DBusException

from wicd import wpath
from wicd import misc
from wicd import dbusmanager

misc.RenameProcess("wicd-monitor")

if __name__ == '__main__':
    wpath.chdir(__file__)

dbusmanager.connect_to_dbus()
dbus_dict = dbusmanager.get_dbus_ifaces()
daemon = dbus_dict["daemon"]
wired = dbus_dict["wired"]
wireless = dbus_dict["wireless"]

mainloop = None

def diewithdbus(func):
    def wrapper(self, *__args, **__kargs):
        try:
            ret = func(self, *__args, **__kargs)
            self.__lost_dbus_count = 0
            return ret
        except DBusException, e:
            print  "Caught exception %s" % str(e)
            if not hasattr(self, "__lost_dbus_count"):
                self.__lost_dbus_count = 0
            if self.__lost_dbus_count > 3:
                mainloop.quit()
            self.__lost_dbus_count += 1
            return True

    wrapper.__name__ = func.__name__
    wrapper.__dict__ = func.__dict__
    wrapper.__doc__ = func.__doc__
    return wrapper 

class ConnectionStatus(object):
    """ Class for monitoring the computer's connection status. """
    def __init__(self):
        """ Initialize variables needed for the connection status methods. """
        self.last_strength = -2
        self.last_state = misc.NOT_CONNECTED
        self.last_reconnect_time = time.time()
        self.last_network = ""
        self.displayed_strength = -1
        self.still_wired = False
        self.network = ''
        self.tried_reconnect = False
        self.connection_lost_counter = 0
        self.reconnecting = False
        self.reconnect_tries = 0
        self.signal_changed = False
        self.iwconfig = ""
        self.trigger_reconnect = False
        self.__lost_dbus_count = 0
        self._to_time = daemon.GetBackendUpdateInterval()
        
        self.add_poll_callback()
        bus = dbusmanager.get_bus()
        bus.add_signal_receiver(self._force_update_connection_status, 
                                "UpdateState", "org.wicd.daemon")
        bus.add_signal_receiver(self._update_timeout_interval,
                                "SignalBackendChanged", "org.wicd.daemon")

    def _update_timeout_interval(self, interval):
        """ Update the callback interval when signaled by the daemon. """
        self._to_time = interval
        gobject.source_remove(self.update_callback)
        self.add_poll_callback()

    def _force_update_connection_status(self):
        """ Run a connection status update on demand.

        Removes the scheduled update_connection_status()
        call, explicitly calls the function, and reschedules
        it.

        """
        gobject.source_remove(self.update_callback)
        self.update_connection_status()
        self.add_poll_callback()
        
    def add_poll_callback(self):
        """ Registers a polling call at a predetermined interval.
        
        The polling interval is determined by the backend in use.
        
        """
        self.update_callback = misc.timeout_add(self._to_time,
                                                self.update_connection_status)
    
    def check_for_wired_connection(self, wired_ip):
        """ Checks for a wired connection.

        Checks for two states:
        1) A wired connection is not in use, but a cable is plugged
           in, and the user has chosen to switch to a wired connection
           whenever its available, even if already connected to a
           wireless network.

        2) A wired connection is currently active.

        """
        self.trigger_reconnect = False
        if not wired_ip and daemon.GetPreferWiredNetwork():
            if not daemon.GetForcedDisconnect() and wired.CheckPluggedIn():
                self.trigger_reconnect = True

        elif wired_ip and wired.CheckPluggedIn():
            # Only change the interface if it's not already set for wired
            if not self.still_wired:
                daemon.SetCurrentInterface(daemon.GetWiredInterface())
                self.still_wired = True
            return True
        # Wired connection isn't active
        elif wired_ip and self.still_wired:
            # If we still have an IP, but no cable is plugged in 
            # we should disconnect to clear it.
            wired.DisconnectWired()
        self.still_wired = False
        return False

    def check_for_wireless_connection(self, wireless_ip):
        """ Checks for an active wireless connection.

        Checks for an active wireless connection.  Also notes
        if the signal strength is 0, and if it remains there
        for too long, triggers a wireless disconnect.

        Returns True if wireless connection is active, and 
        False otherwise.

        """

        # Make sure we have an IP before we do anything else.
        if not wireless_ip:
            return False

        if daemon.NeedsExternalCalls():
            self.iwconfig = wireless.GetIwconfig()
        else:
            self.iwconfig = ''
        # Reset this, just in case.
        self.tried_reconnect = False
        bssid = wireless.GetApBssid()
        if not bssid:
            return False

        wifi_signal = self._get_printable_sig_strength(always_positive=True)
        if wifi_signal <= 0:
            # If we have no signal, increment connection loss counter.
            # If we haven't gotten any signal 4 runs in a row (12 seconds),
            # try to reconnect.
            self.connection_lost_counter += 1
            print self.connection_lost_counter
            if self.connection_lost_counter >= 4 and daemon.GetAutoReconnect():
                wireless.DisconnectWireless()
                self.connection_lost_counter = 0
                return False
        else:  # If we have a signal, reset the counter
            self.connection_lost_counter = 0

        if (wifi_signal != self.last_strength or
            self.network != self.last_network):
            self.last_strength = wifi_signal
            self.last_network = self.network
            self.signal_changed = True
            daemon.SetCurrentInterface(daemon.GetWirelessInterface())    

        return True

    @diewithdbus
    def update_connection_status(self):
        """ Updates the tray icon and current connection status.

        Determines the current connection state and sends a dbus signal
        announcing when the status changes.  Also starts the automatic
        reconnection process if necessary.

        """
        wired_ip = None
        wifi_ip = None

        if daemon.GetSuspend():
            print "Suspended."
            state = misc.SUSPENDED
            return self.update_state(state)

        # Determine what our current state is.
        # Are we currently connecting?
        if daemon.CheckIfConnecting():
            state = misc.CONNECTING
            return self.update_state(state)

        daemon.SendConnectResultsIfAvail()

        # Check for wired.
        wired_ip = wired.GetWiredIP("")
        wired_found = self.check_for_wired_connection(wired_ip)
        if wired_found:
            return self.update_state(misc.WIRED, wired_ip=wired_ip)

        # Check for wireless
        wifi_ip = wireless.GetWirelessIP("")
        self.signal_changed = False
        wireless_found = self.check_for_wireless_connection(wifi_ip)
        if wireless_found:
            if self.trigger_reconnect:
                # If we made it here, that means we want to switch
                # to a wired network whenever possible, but a wireless
                # connection is active.  So we kill the wireless connection
                # so the autoconnect logic will connect to the wired network.
                self.trigger_reconnect = False

                # Don't trigger it if the gui is open, because autoconnect
                # is disabled while it's open.
                if not daemon.GetGUIOpen():
                    print 'Killing wireless connection to switch to wired...'
                    wireless.DisconnectWireless()
                    daemon.AutoConnect(False, reply_handler=lambda *a:None,
                                       error_handler=lambda *a:None)
                    return self.update_state(misc.NOT_CONNECTED)
            return self.update_state(misc.WIRELESS, wifi_ip=wifi_ip)

        state = misc.NOT_CONNECTED
        if self.last_state == misc.WIRELESS:
            from_wireless = True
        else:
            from_wireless = False
            self.auto_reconnect(from_wireless)
        return self.update_state(state)

    def update_state(self, state, wired_ip=None, wifi_ip=None):
        """ Set the current connection state. """
        # Set our connection state/info.
        iwconfig = self.iwconfig
        if state == misc.NOT_CONNECTED:
            info = [""]
        elif state == misc.SUSPENDED:
            info = [""]
        elif state == misc.CONNECTING:
            if wired.CheckIfWiredConnecting():
                info = ["wired"]
            else:
                info = ["wireless", wireless.GetCurrentNetwork(iwconfig)]
        elif state == misc.WIRELESS:
            self.reconnect_tries = 0
            info = [str(wifi_ip), wireless.GetCurrentNetwork(iwconfig),
                    str(self._get_printable_sig_strength()),
                    str(wireless.GetCurrentNetworkID(iwconfig)),
                    wireless.GetCurrentBitrate(iwconfig)]
        elif state == misc.WIRED:
            self.reconnect_tries = 0
            info = [str(wired_ip)]
        else:
            print 'ERROR: Invalid state!'
            return True

        daemon.SetConnectionStatus(state, info)

        # Send a D-Bus signal announcing status has changed if necessary.
        if (state != self.last_state or (state == misc.WIRELESS and 
                                         self.signal_changed)):
            daemon.EmitStatusChanged(state, info)

        if (state != self.last_state) and (state == misc.NOT_CONNECTED) and \
            (not daemon.GetForcedDisconnect()):
            daemon.Disconnect()
            # Disconnect() sets forced disconnect = True
            # so we'll revert that
            daemon.SetForcedDisconnect(False)
        self.last_state = state
        return True

    def _get_printable_sig_strength(self, always_positive=False):
        """ Get the correct signal strength format. """
        try:
            if daemon.GetSignalDisplayType() == 0:
                wifi_signal = int(wireless.GetCurrentSignalStrength(self.iwconfig))
            else:
                if always_positive:
                    # because dBm is negative, add 99 to the signal. This way, if
                    # the signal drops below -99, wifi_signal will == 0, and
                    # an automatic reconnect will be triggered
                    # this is only used in check_for_wireless_connection
                    wifi_signal = 99 + int(wireless.GetCurrentDBMStrength(self.iwconfig))
                else:
                    wifi_signal = int(wireless.GetCurrentDBMStrength(self.iwconfig))
        except TypeError:
            wifi_signal = 0        

        return wifi_signal

    def auto_reconnect(self, from_wireless=None):
        """ Automatically reconnects to a network if needed.

        If automatic reconnection is turned on, this method will
        attempt to first reconnect to the last used wireless network, and
        should that fail will simply run AutoConnect()

        """
        if self.reconnecting:
            return

        # Some checks to keep reconnect retries from going crazy.
        if (self.reconnect_tries > 3 and
            (time.time() - self.last_reconnect_time) < 200):
            print "Throttling autoreconnect"
            return

        self.reconnecting = True
        daemon.SetCurrentInterface('')

        if daemon.ShouldAutoReconnect():
            print 'Starting automatic reconnect process'
            self.last_reconnect_time = time.time()
            self.reconnect_tries += 1

            # If we just lost a wireless connection, try to connect to that
            # network again.  Otherwise just call Autoconnect.
            cur_net_id = wireless.GetCurrentNetworkID(self.iwconfig)
            if from_wireless and cur_net_id > -1:
                # make sure disconnect scripts are run
                # before we reconnect
                print 'Disconnecting from network'
                wireless.DisconnectWireless()
                print 'Trying to reconnect to last used wireless ' + \
                      'network'
                wireless.ConnectWireless(cur_net_id)
            else:
                daemon.AutoConnect(True, reply_handler=reply_handle,
                                   error_handler=err_handle)
        self.reconnecting = False

def reply_handle():
    """ Just a dummy function needed for asynchronous dbus calls. """
    pass

def err_handle(error):
    """ Just a dummy function needed for asynchronous dbus calls. """
    pass

def main():
    """ Starts the connection monitor. 

    Starts a ConnectionStatus instance, sets the status to update
    an amount of time determined by the active backend.

    """
    global mainloop
    monitor = ConnectionStatus()
    mainloop = gobject.MainLoop()
    mainloop.run()


if __name__ == '__main__':
    main()



""" netentry -- Network entry widgets for the GUI.

This module provides GUI widgets used to represent wired and wireless
entries in the GUI's network list, as well as any settings dialogs
contained within them.

"""
#
#   Copyright (C) 2008-2009 Adam Blackburn
#   Copyright (C) 2008-2009 Dan O'Reilly
#   Copyright (C) 2009      Andrew Psaltis
#   Copyright (C) 2011      David Paleino
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import os

import wicd.misc as misc
import wicd.wpath as wpath
import wicd.dbusmanager as dbusmanager
from wicd.misc import noneToString, stringToNone, noneToBlankString, to_bool
from guiutil import error, LabelEntry, GreyLabel, LeftAlignedLabel, string_input, ProtectedLabelEntry

from wicd.translations import language, _

# These get set when a NetworkEntry is instantiated.
daemon = None
wired = None
wireless = None
        
def setup_dbus():
    global daemon, wireless, wired
    daemon = dbusmanager.get_interface('daemon')
    wireless = dbusmanager.get_interface('wireless')
    wired = dbusmanager.get_interface('wired')
    
class AdvancedSettingsDialog(gtk.Dialog):
    def __init__(self, network_name=None):
        """ Build the base advanced settings dialog.
        
        This class isn't used by itself, instead it is used as a parent for
        the WiredSettingsDialog and WirelessSettingsDialog.
        
        """
        # if no network name was passed, just use Properties as the title
        if network_name:
            title = '%s - %s' % (network_name, _('Properties'))
        else:
            title = _('Properties')

        gtk.Dialog.__init__(self, title=title,
                            flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_CANCEL,
                                                           gtk.RESPONSE_REJECT,
                                                           gtk.STOCK_OK,
                                                           gtk.RESPONSE_ACCEPT))

        self.set_default_size()

        self.connect('show', lambda *a, **k: self.set_default_size())

        # Set up the Advanced Settings Dialog.
        self.txt_ip = LabelEntry(_('IP'))
        self.txt_ip.entry.connect('focus-out-event', self.set_defaults)
        self.txt_netmask = LabelEntry(_('Netmask'))
        self.txt_gateway = LabelEntry(_('Gateway'))
        self.txt_search_dom = LabelEntry(_('Search domain'))
        self.txt_domain = LabelEntry(_('DNS domain'))
        self.txt_dns_1 = LabelEntry(_('DNS server') + ' 1')
        self.txt_dns_2 = LabelEntry(_('DNS server') + ' 2')
        self.txt_dns_3 = LabelEntry(_('DNS server') + ' 3')
        dhcp_hostname_hbox = gtk.HBox(False, 0)
        self.chkbox_use_dhcp_hostname = gtk.CheckButton()
        self.txt_dhcp_hostname = LabelEntry("DHCP Hostname")
        dhcp_hostname_hbox.pack_start(self.chkbox_use_dhcp_hostname, fill=False, expand=False)
        dhcp_hostname_hbox.pack_start(self.txt_dhcp_hostname)
        self.chkbox_static_ip = gtk.CheckButton(_('Use Static IPs'))
        self.chkbox_static_dns = gtk.CheckButton(_('Use Static DNS'))
        self.chkbox_global_dns = gtk.CheckButton(_('Use global DNS servers'))
        self.hbox_dns = gtk.HBox(False, 0)
        self.hbox_dns.pack_start(self.chkbox_static_dns)
        self.hbox_dns.pack_start(self.chkbox_global_dns)
        
        # Set up the script settings button
        self.script_button = gtk.Button()
        script_image = gtk.Image()
        script_image.set_from_stock(gtk.STOCK_EXECUTE, 4)
        script_image.set_padding(4, 0)
        #self.script_button.set_alignment(.5, .5)
        self.script_button.set_image(script_image)
        self.script_button.set_label(_('Scripts'))
        
        self.button_hbox = gtk.HBox(False, 2)
        self.button_hbox.pack_start(self.script_button, fill=False, expand=False)
        self.button_hbox.show()

        self.swindow = gtk.ScrolledWindow()
        self.swindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.viewport = gtk.Viewport()
        self.viewport.set_shadow_type(gtk.SHADOW_NONE)
        self.cvbox = gtk.VBox()
        self.viewport.add(self.cvbox)
        self.swindow.add(self.viewport)
        self.vbox.pack_start(self.swindow)
        
        assert(isinstance(self.cvbox, gtk.VBox))
        self.cvbox.pack_start(self.chkbox_static_ip, fill=False, expand=False)
        self.cvbox.pack_start(self.txt_ip, fill=False, expand=False)
        self.cvbox.pack_start(self.txt_netmask, fill=False, expand=False)
        self.cvbox.pack_start(self.txt_gateway, fill=False, expand=False)
        self.cvbox.pack_start(self.hbox_dns, fill=False, expand=False)
        self.cvbox.pack_start(self.txt_domain, fill=False, expand=False)
        self.cvbox.pack_start(self.txt_search_dom, fill=False, expand=False)
        self.cvbox.pack_start(self.txt_dns_1, fill=False, expand=False)
        self.cvbox.pack_start(self.txt_dns_2, fill=False, expand=False)
        self.cvbox.pack_start(self.txt_dns_3, fill=False, expand=False)
        self.cvbox.pack_start(dhcp_hostname_hbox, fill=False, expand=False)
        self.cvbox.pack_end(self.button_hbox, fill=False, expand=False, padding=5)
        
        # Connect the events to the actions
        self.chkbox_static_ip.connect("toggled", self.toggle_ip_checkbox)
        self.chkbox_static_dns.connect("toggled", self.toggle_dns_checkbox)
        self.chkbox_global_dns.connect("toggled", self.toggle_global_dns_checkbox)
        self.chkbox_use_dhcp_hostname.connect('toggled',
                                              self.toggle_dhcp_hostname_checkbox)
        
        # Start with all disabled, then they will be enabled later.
        self.chkbox_static_ip.set_active(False)
        self.chkbox_static_dns.set_active(False)


    def set_default_size(self):
        width, height = self.get_size()
        s_height = gtk.gdk.screen_height()
        if s_height < 768:
            height = s_height * .75
        else:
            height = 600
        self.resize(int(width), int(height))
        
    def set_defaults(self, widget=None, event=None):
        """ Put some default values into entries to help the user out. """
        self.txt_ip.set_text(self.txt_ip.get_text().strip())
        ipAddress = self.txt_ip.get_text()  # For easy typing :)
        netmask = self.txt_netmask
        gateway = self.txt_gateway
        ip_parts = misc.IsValidIP(ipAddress)
        if ip_parts:
            if stringToNone(gateway.get_text()) is None:  # Make sure the gateway box is blank
                # Fill it in with a .1 at the end
                gateway.set_text('.'.join(ip_parts[0:3]) + '.1')

            if stringToNone(netmask.get_text()) is None:  # Make sure the netmask is blank
                netmask.set_text('255.255.255.0')  # Fill in the most common one
        elif ipAddress != "":
            error(None, _('Invalid IP address entered.'))

    def reset_static_checkboxes(self):
        # Enable the right stuff
        if stringToNone(self.txt_ip.get_text()):
            self.chkbox_static_ip.set_active(True)
            self.chkbox_static_dns.set_active(True)
            self.chkbox_static_dns.set_sensitive(False)
        else:
            self.chkbox_static_ip.set_active(False)
            self.chkbox_static_dns.set_sensitive(True)

        if stringToNone(self.txt_dns_1.get_text()) or \
           self.chkbox_global_dns.get_active():
            self.chkbox_static_dns.set_active(True)
        else:
            self.chkbox_static_dns.set_active(False)

        # This will properly disable unused boxes.
        self.toggle_ip_checkbox()
        self.toggle_dns_checkbox()
        self.toggle_global_dns_checkbox()

    def toggle_ip_checkbox(self, widget=None):
        """Toggle entries/checkboxes based on the static IP checkbox. """
        # Should disable the static IP text boxes, and also enable the DNS
        # checkbox when disabled and disable when enabled.
        if self.chkbox_static_ip.get_active():
            self.chkbox_static_dns.set_active(True)
            self.chkbox_static_dns.set_sensitive(False)
        else:
            self.chkbox_static_dns.set_sensitive(True)

        self.txt_ip.set_sensitive(self.chkbox_static_ip.get_active())
        self.txt_netmask.set_sensitive(self.chkbox_static_ip.get_active())
        self.txt_gateway.set_sensitive(self.chkbox_static_ip.get_active())

    def toggle_dns_checkbox(self, widget=None):
        """ Toggle entries and checkboxes based on the static dns checkbox. """
        # Should disable the static DNS boxes
        if self.chkbox_static_ip.get_active():
            self.chkbox_static_dns.set_active(True)
            self.chkbox_static_dns.set_sensitive(False)

        self.chkbox_global_dns.set_sensitive(self.chkbox_static_dns.
                                                            get_active())
        
        l = [self.txt_dns_1, self.txt_dns_2, self.txt_dns_3, self.txt_domain,
             self.txt_search_dom]
        if self.chkbox_static_dns.get_active():
            # If global dns is on, don't use local dns
            for w in l:
                w.set_sensitive(not self.chkbox_global_dns.get_active())
        else:
            for w in l:
                w.set_sensitive(False)
            self.chkbox_global_dns.set_active(False)

    def toggle_dhcp_hostname_checkbox(self, widget=None):
        self.txt_dhcp_hostname.set_sensitive(
            self.chkbox_use_dhcp_hostname.get_active())

    def toggle_global_dns_checkbox(self, widget=None):
        """ Set the DNS entries' sensitivity based on the Global checkbox. """
        global_dns_active = daemon.GetUseGlobalDNS()
        if not global_dns_active and self.chkbox_global_dns.get_active():
            error(None, _('Global DNS has not been enabled in general preferences.'))
            self.chkbox_global_dns.set_active(False)
        if daemon.GetUseGlobalDNS() and self.chkbox_static_dns.get_active():
            for w in [self.txt_dns_1, self.txt_dns_2, self.txt_dns_3, 
                      self.txt_domain, self.txt_search_dom]:
                w.set_sensitive(not self.chkbox_global_dns.get_active())
    
    def toggle_encryption(self, widget=None):
        """ Toggle the encryption combobox based on the encryption checkbox. """
        active = self.chkbox_encryption.get_active()
        self.vbox_encrypt_info.set_sensitive(active)
        self.combo_encryption.set_sensitive(active)
                
    def destroy_called(self, *args):
        """ Clean up everything. """
        super(AdvancedSettingsDialog, self).destroy()
        self.destroy()
        del self

    def save_settings(self):
        """ Save settings common to wired and wireless settings dialogs. """
        if self.chkbox_static_ip.get_active():
            self.set_net_prop("ip", noneToString(self.txt_ip.get_text()))
            self.set_net_prop("netmask", noneToString(self.txt_netmask.get_text()))
            self.set_net_prop("gateway", noneToString(self.txt_gateway.get_text()))
        else:
            self.set_net_prop("ip", '')
            self.set_net_prop("netmask", '')
            self.set_net_prop("gateway", '')

        if self.chkbox_static_dns.get_active() and \
           not self.chkbox_global_dns.get_active():
            self.set_net_prop('use_static_dns', True)
            self.set_net_prop('use_global_dns', False)
            self.set_net_prop('dns_domain', noneToString(self.txt_domain.get_text()))
            self.set_net_prop("search_domain", noneToString(self.txt_search_dom.get_text()))
            self.set_net_prop("dns1", noneToString(self.txt_dns_1.get_text()))
            self.set_net_prop("dns2", noneToString(self.txt_dns_2.get_text()))
            self.set_net_prop("dns3", noneToString(self.txt_dns_3.get_text()))
        elif self.chkbox_static_dns.get_active() and \
             self.chkbox_global_dns.get_active():
            self.set_net_prop('use_static_dns', True)
            self.set_net_prop('use_global_dns', True)
        else:
            self.set_net_prop('use_static_dns', False)
            self.set_net_prop('use_global_dns', False)
            self.set_net_prop('dns_domain', '')
            self.set_net_prop("search_domain", '')
            self.set_net_prop("dns1", '')
            self.set_net_prop("dns2", '')
            self.set_net_prop("dns3", '')
        self.set_net_prop('usedhcphostname',
                          self.chkbox_use_dhcp_hostname.get_active())
        self.set_net_prop("dhcphostname",noneToString(self.txt_dhcp_hostname.get_text()))

    def change_encrypt_method(self, widget=None):
        """ Load all the entries for a given encryption method. """
        for z in self.vbox_encrypt_info:
            z.destroy()  # Remove stuff in there already
        ID = self.combo_encryption.get_active()
        methods = self.encrypt_types
        self.encryption_info = {}
        
        # If nothing is selected, select the first entry.
        if ID == -1:
            self.combo_encryption.set_active(0)
            ID = 0

        for type_ in ['required', 'optional']:
            fields = methods[ID][type_]
            for field in fields:
                try:
                    field_text = language[field[1].lower().replace(' ','_')]
                except KeyError:
                    field_text = field[1].replace(' ','_')

                if field in methods[ID]['protected']:
                    box = ProtectedLabelEntry(field_text)
                else:
                    box = LabelEntry(field_text)

                self.vbox_encrypt_info.pack_start(box)
                # Add the data to a dict, so that the information
                # can be easily accessed by giving the name of the wanted
                # data.
                self.encryption_info[field[0]] = [box, type_]
                
                if self.wired:
                    box.entry.set_text(noneToBlankString(
                        wired.GetWiredProperty(field[0])))
                else:
                    box.entry.set_text(noneToBlankString(
                        wireless.GetWirelessProperty(self.networkID, field[0])))
        self.vbox_encrypt_info.show_all()

        
class WiredSettingsDialog(AdvancedSettingsDialog):
    def __init__(self, name):
        """ Build the wired settings dialog. """
        AdvancedSettingsDialog.__init__(self, _('Wired Network'))
        
        # So we can test if we are wired or wireless (for change_encrypt_method())
        self.wired = True
        
        ## This section is largely copied from WirelessSettingsDialog, but with some changes
        # Set up encryption stuff
        self.combo_encryption = gtk.combo_box_new_text()
        self.chkbox_encryption = gtk.CheckButton(_('Use Encryption'))
        # Make the vbox to hold the encryption stuff.
        self.vbox_encrypt_info = gtk.VBox(False, 0)
        self.chkbox_encryption.set_active(bool(wired.GetWiredProperty('encryption_enabled')))
        self.combo_encryption.set_sensitive(False)
        self.encrypt_types = misc.LoadEncryptionMethods(wired = True)
 
        # Build the encryption menu
        for x, enc_type in enumerate(self.encrypt_types):
            self.combo_encryption.append_text(enc_type['name'])
        self.combo_encryption.set_active(0)
        self.change_encrypt_method()
        self.toggle_encryption()

        self.cvbox.pack_start(self.chkbox_encryption, False, False)
        self.cvbox.pack_start(self.combo_encryption, False, False)
        self.cvbox.pack_start(self.vbox_encrypt_info, False, False)
        
        # Connect signals.
        self.chkbox_encryption.connect("toggled", self.toggle_encryption)
        self.combo_encryption.connect("changed", self.change_encrypt_method)
        
        self.des = self.connect("destroy", self.destroy_called)
        self.script_button.connect("clicked", self.edit_scripts)
        self.prof_name = name
        
    def set_net_prop(self, option, value):
        """ Sets the given option to the given value for this network. """
        wired.SetWiredProperty(option, value)
        
    def edit_scripts(self, widget=None, event=None):
        """ Launch the script editting dialog. """
        profile = self.prof_name
        cmdend = [os.path.join(wpath.gtk, "configscript.py"), profile, "wired"]
        if os.getuid() != 0:
            cmdbase = misc.get_sudo_cmd(_('You must enter your password to configure scripts'),
                                        prog_num=daemon.GetSudoApp())
            if not cmdbase:
                error(None, _('Could not find a graphical sudo program. '\
                              'The script editor could not be launched.  '\
                              "You'll have to edit scripts directly your configuration file."))
                return
            cmdbase.extend(cmdend)
            misc.LaunchAndWait(cmdbase)
        else:
            misc.LaunchAndWait(cmdend)
        
    def set_values(self):
        """ Fill in the Gtk.Entry objects with the correct values. """
        self.txt_ip.set_text(self.format_entry("ip"))
        self.txt_netmask.set_text(self.format_entry("netmask"))
        self.txt_gateway.set_text(self.format_entry("gateway"))

        self.txt_dns_1.set_text(self.format_entry("dns1"))
        self.txt_dns_2.set_text(self.format_entry("dns2"))
        self.txt_dns_3.set_text(self.format_entry("dns3"))
        self.txt_domain.set_text(self.format_entry("dns_domain"))
        self.txt_search_dom.set_text(self.format_entry("search_domain"))
        self.chkbox_global_dns.set_active(bool(wired.GetWiredProperty("use_global_dns")))

        dhcphname = wired.GetWiredProperty("dhcphostname")
        if dhcphname is None:
            dhcphname = os.uname()[1]

        self.txt_dhcp_hostname.set_text(dhcphname)
        self.reset_static_checkboxes()
        
        self.chkbox_encryption.set_active(bool(wired.GetWiredProperty('encryption_enabled')))
        self.change_encrypt_method()
        self.toggle_encryption()
        
    def save_settings(self):
        # Check encryption info
        encrypt_info = self.encryption_info
        self.set_net_prop("encryption_enabled", self.chkbox_encryption.get_active())
        if self.chkbox_encryption.get_active():
            print "setting encryption info..."
            encrypt_methods = self.encrypt_types
            self.set_net_prop("enctype",
                               encrypt_methods[self.combo_encryption.get_active()]['type'])
            # Make sure all required fields are filled in.
            for entry_info in encrypt_info.itervalues():
                if entry_info[0].entry.get_text() == "" and \
                   entry_info[1] == 'required':
                    error(self, "%s (%s)" % (_('Required encryption information is missing.'),
                                             entry_info[0].label.get_label())
                          )
                    return False
            # Now save all the entries.
            for entry_key, entry_info in encrypt_info.iteritems():
                self.set_net_prop(entry_key, 
                                  noneToString(entry_info[0].entry.get_text()))
        elif not wired and not self.chkbox_encryption.get_active() and \
             wireless.GetWirelessProperty(networkid, "encryption"):
            # Encrypt checkbox is off, but the network needs it.
            error(self, _('This network requires encryption to be enabled.'))
            return False
        else:
            print "no encryption specified..."
            self.set_net_prop("enctype", "None")
        AdvancedSettingsDialog.save_settings(self)
        wired.SaveWiredNetworkProfile(self.prof_name)
        return True

    def format_entry(self, label):
        """ Helper method to fetch and format wired properties. """
        return noneToBlankString(wired.GetWiredProperty(label))
    
    def destroy_called(self, *args):
        """ Clean up everything. """
        self.disconnect(self.des)
        super(WiredSettingsDialog, self).destroy_called()
        self.destroy()
        del self

        
class WirelessSettingsDialog(AdvancedSettingsDialog):
    def __init__(self, networkID):
        """ Build the wireless settings dialog. """
        AdvancedSettingsDialog.__init__(self, wireless.GetWirelessProperty(networkID, 'essid'))
        # So we can test if we are wired or wireless (for change_encrypt_method())
        self.wired = False
        
        # Set up encryption stuff
        self.networkID = networkID
        self.combo_encryption = gtk.combo_box_new_text()
        self.chkbox_encryption = gtk.CheckButton(_('Use Encryption'))
        self.chkbox_global_settings = gtk.CheckButton(_('Use these settings for all networks sharing this essid'))
        # Make the vbox to hold the encryption stuff.
        self.vbox_encrypt_info = gtk.VBox(False, 0)        
        self.toggle_encryption()
        self.chkbox_encryption.set_active(False)
        self.combo_encryption.set_sensitive(False)
        self.encrypt_types = misc.LoadEncryptionMethods()
 
        information_button = gtk.Button(stock=gtk.STOCK_INFO)
        self.button_hbox.pack_start(information_button, False, False)
        information_button.connect('clicked', lambda *a, **k: WirelessInformationDialog(networkID, self))
        information_button.show()
        
        # Build the encryption menu
        activeID = -1  # Set the menu to this item when we are done
        for x, enc_type in enumerate(self.encrypt_types):
            self.combo_encryption.append_text(enc_type['name'])
            if enc_type['type'] == wireless.GetWirelessProperty(networkID, "enctype"):
                activeID = x
        self.combo_encryption.set_active(activeID)
        if activeID != -1:
            self.chkbox_encryption.set_active(True)
            self.combo_encryption.set_sensitive(True)
            self.vbox_encrypt_info.set_sensitive(True)
        else:
            self.combo_encryption.set_active(0)
        self.change_encrypt_method()

        self.cvbox.pack_start(self.chkbox_global_settings, False, False)
        self.cvbox.pack_start(self.chkbox_encryption, False, False)
        self.cvbox.pack_start(self.combo_encryption, False, False)
        self.cvbox.pack_start(self.vbox_encrypt_info, False, False)
        
        # Connect signals.
        self.chkbox_encryption.connect("toggled", self.toggle_encryption)
        self.combo_encryption.connect("changed", self.change_encrypt_method)
        self.script_button.connect("clicked", self.edit_scripts)
        self.des = self.connect("destroy", self.destroy_called)

    def destroy_called(self, *args):
        """ Clean up everything. """
        self.disconnect(self.des)
        super(WirelessSettingsDialog, self).destroy_called()
        self.destroy()
        del self
        
    def edit_scripts(self, widget=None, event=None):
        """ Launch the script editting dialog. """
        cmdend = [os.path.join(wpath.gtk, "configscript.py"),
                                str(self.networkID), "wireless"]
        if os.getuid() != 0:
            cmdbase = misc.get_sudo_cmd(_('You must enter your password to configure scripts'),
                                        prog_num=daemon.GetSudoApp())
            if not cmdbase:
                error(None, _('Could not find a graphical sudo program. '\
                              'The script editor could not be launched.  '\
                              "You'll have to edit scripts directly your configuration file."))
                return
            cmdbase.extend(cmdend)
            misc.LaunchAndWait(cmdbase)
        else:
            misc.LaunchAndWait(cmdend)
        
    def set_net_prop(self, option, value):
        """ Sets the given option to the given value for this network. """
        wireless.SetWirelessProperty(self.networkID, option, value)
        
    def set_values(self):
        """ Set the various network settings to the right values. """
        networkID = self.networkID
        self.txt_ip.set_text(self.format_entry(networkID,"ip"))
        self.txt_netmask.set_text(self.format_entry(networkID,"netmask"))
        self.txt_gateway.set_text(self.format_entry(networkID,"gateway"))

        self.chkbox_global_dns.set_active(bool(wireless.GetWirelessProperty(networkID,
                                                                  'use_global_dns')))
        self.chkbox_static_dns.set_active(bool(wireless.GetWirelessProperty(networkID,
                                                                  'use_static_dns')))
        
        self.txt_dns_1.set_text(self.format_entry(networkID, "dns1"))
        self.txt_dns_2.set_text(self.format_entry(networkID, "dns2"))
        self.txt_dns_3.set_text(self.format_entry(networkID, "dns3"))
        self.txt_domain.set_text(self.format_entry(networkID, "dns_domain"))
        self.txt_search_dom.set_text(self.format_entry(networkID, "search_domain"))
        
        self.reset_static_checkboxes()
        self.chkbox_encryption.set_active(bool(wireless.GetWirelessProperty(networkID,
                                                                       'encryption')))
        self.chkbox_global_settings.set_active(bool(wireless.GetWirelessProperty(networkID,
                                                             'use_settings_globally')))

        self.chkbox_use_dhcp_hostname.set_active(
            bool(wireless.GetWirelessProperty(networkID, 'usedhcphostname')))


        dhcphname = wireless.GetWirelessProperty(networkID,"dhcphostname")
        if dhcphname is None:
            dhcphname = os.uname()[1]
        self.txt_dhcp_hostname.set_text(dhcphname)

        self.toggle_dhcp_hostname_checkbox()

        activeID = -1  # Set the menu to this item when we are done
        user_enctype = wireless.GetWirelessProperty(networkID, "enctype")
        for x, enc_type in enumerate(self.encrypt_types):
            if enc_type['type'] == user_enctype:
                activeID = x
        
        self.combo_encryption.set_active(activeID)
        if activeID != -1:
            self.chkbox_encryption.set_active(True)
            self.combo_encryption.set_sensitive(True)
            self.vbox_encrypt_info.set_sensitive(True)
        else:
            self.combo_encryption.set_active(0)
        self.change_encrypt_method()
        
    def save_settings(self, networkid):
        # Check encryption info
        encrypt_info = self.encryption_info
        if self.chkbox_encryption.get_active():
            print "setting encryption info..."
            encrypt_methods = self.encrypt_types
            self.set_net_prop("enctype",
                               encrypt_methods[self.combo_encryption.get_active()]['type'])
            # Make sure all required fields are filled in.
            for entry_info in encrypt_info.itervalues():
                if entry_info[0].entry.get_text() == "" and \
                   entry_info[1] == 'required':
                    error(self, "%s (%s)" % (_('Required encryption information is missing.'),
                                             entry_info[0].label.get_label())
                          )
                    return False
            # Now save all the entries.
            for entry_key, entry_info in encrypt_info.iteritems():
                self.set_net_prop(entry_key, 
                                  noneToString(entry_info[0].entry.get_text()))
        elif not self.chkbox_encryption.get_active() and \
             wireless.GetWirelessProperty(networkid, "encryption"):
            # Encrypt checkbox is off, but the network needs it.
            error(self, _('This network requires encryption to be enabled.'))
            return False
        else:
            print "no encryption specified..."
            self.set_net_prop("enctype", "None")
        AdvancedSettingsDialog.save_settings(self)
        
        if self.chkbox_global_settings.get_active():
            self.set_net_prop('use_settings_globally', True)
        else:
            self.set_net_prop('use_settings_globally', False)
            wireless.RemoveGlobalEssidEntry(networkid)
            
        wireless.SaveWirelessNetworkProfile(networkid)
        return True

    def format_entry(self, networkid, label):
        """ Helper method for fetching/formatting wireless properties. """
        return noneToBlankString(wireless.GetWirelessProperty(networkid, label))
        
        
class NetworkEntry(gtk.HBox):
    def __init__(self):
        """ Base network entry class.
        
        Provides gtk objects used by both the WiredNetworkEntry and
        WirelessNetworkEntry classes.
        
        """
        setup_dbus()
        gtk.HBox.__init__(self, False, 2)
        self.image = gtk.Image()
        self.pack_start(self.image, False, False)

        # Create an HBox to hold the buttons
        self.buttons_hbox = gtk.HBox(False, 6)
        
        # Set up the Connect button
        self.connect_button = gtk.Button(stock=gtk.STOCK_CONNECT)
        self.connect_hbox = gtk.HBox(False, 2)
        self.connect_hbox.pack_start(self.connect_button, False, False)
        self.connect_hbox.show()
        
        # Set up the Disconnect button
        self.disconnect_button = gtk.Button(stock=gtk.STOCK_DISCONNECT)
        self.connect_hbox.pack_start(self.disconnect_button, False, False)

        # Create a label to hold the name of the entry
        self.name_label = gtk.Label()
        self.name_label.set_alignment(0, 0.5)
        
        # Set up the VBox that goes in the gtk.Expander
        self.expander_vbox = gtk.VBox(False, 1)
        self.expander_vbox.show()
        self.pack_end(self.expander_vbox)
        
        # Set up the advanced settings button
        self.advanced_button = gtk.Button()
        self.advanced_image = gtk.Image()
        self.advanced_image.set_from_stock(gtk.STOCK_EDIT, 4)
        self.advanced_image.set_padding(4, 0)
        self.advanced_button.set_alignment(.5, .5)
        self.advanced_button.set_label(_('Properties'))
        self.advanced_button.set_image(self.advanced_image)
        
        self.buttons_hbox.pack_start(self.connect_hbox, False, False)
        self.buttons_hbox.pack_start(self.advanced_button, False, False)

        self.vbox_top = gtk.VBox(False, 0)
        self.expander_vbox.pack_start(self.name_label)
        self.expander_vbox.pack_start(self.vbox_top)
        self.expander_vbox.pack_start(self.buttons_hbox)
    
    def destroy_called(self, *args):
        """ Clean up everything. """
        super(NetworkEntry, self).destroy()
        self.destroy()
        del self
        

class WiredNetworkEntry(NetworkEntry):
    def __init__(self):
        """ Load the wired network entry. """
        NetworkEntry.__init__(self)
        # Center the picture and pad it a bit
        self.image.set_padding(0, 0)
        self.image.set_alignment(.5, .5)
        self.image.set_size_request(60, -1)
        self.image.set_from_file(wpath.images + "wired-gui.svg")
        self.image.show()
        self.connect_button.show()

        self.name_label.set_use_markup(True)
        self.name_label.set_label("<b>" + _('Wired Network') + "</b>")
        
        self.is_full_gui = True
        
        self.button_add = gtk.Button(stock=gtk.STOCK_ADD)
        self.button_delete = gtk.Button(stock=gtk.STOCK_DELETE)
        self.profile_help = gtk.Label(_('To connect to a wired network, you must create a network profile. To create a network profile, type a name that describes this network, and press Add.'))
        self.chkbox_default_profile = gtk.CheckButton(_('Use as default profile (overwrites any previous default)'))
        self.combo_profile_names = gtk.combo_box_new_text() 
        
        # Format the profile help label.
        self.profile_help.set_justify(gtk.JUSTIFY_LEFT)
        self.profile_help.set_line_wrap(True)

        # Pack the various VBox objects.
        self.hbox_temp = gtk.HBox(False, 0)
        self.hbox_def = gtk.HBox(False, 0)
        self.vbox_top.pack_start(self.profile_help, True, True)
        self.vbox_top.pack_start(self.hbox_def)
        self.vbox_top.pack_start(self.hbox_temp)
        self.hbox_temp.pack_start(self.combo_profile_names, True, True)
        self.hbox_temp.pack_start(self.button_add, False, False)
        self.hbox_temp.pack_start(self.button_delete, False, False)
        self.hbox_def.pack_start(self.chkbox_default_profile, False, False)

        # Connect events
        self.button_add.connect("clicked", self.add_profile)
        self.button_delete.connect("clicked", self.remove_profile)
        self.chkbox_default_profile.connect("toggled",
                                            self.toggle_default_profile)
        self.combo_profile_names.connect("changed", self.change_profile)
        
        # Build profile list.
        self.profile_list = wired.GetWiredProfileList()
        default_prof = wired.GetDefaultWiredNetwork()
        if self.profile_list:
            starting_index = 0
            for x, prof in enumerate(self.profile_list):
                self.combo_profile_names.append_text(prof)
                if default_prof == prof:
                    starting_index = x
            self.combo_profile_names.set_active(starting_index)
        else:
            print "no wired profiles found"
            self.profile_help.show()
            
        self.advanced_dialog = WiredSettingsDialog(self.combo_profile_names.get_active_text())            

        # Show everything, but hide the profile help label.
        self.show_all()
        self.profile_help.hide()
        
        # Toggle the default profile checkbox to the correct state.
        if to_bool(wired.GetWiredProperty("default")):
            self.chkbox_default_profile.set_active(True)
        else:
            self.chkbox_default_profile.set_active(False)

        self.check_enable()
        self.wireddis = self.connect("destroy", self.destroy_called)
        
    def destroy_called(self, *args):
        """ Clean up everything. """
        self.disconnect(self.wireddis)
        self.advanced_dialog.destroy_called()
        del self.advanced_dialog
        super(WiredNetworkEntry, self).destroy_called()
        self.destroy()
        del self
        
    def save_wired_settings(self):
        """ Save wired network settings. """
        return self.advanced_dialog.save_settings()

    def check_enable(self):
        """ Disable objects if the profile list is empty. """
        profile_list = wired.GetWiredProfileList()
        if not profile_list:
            self.button_delete.set_sensitive(False)
            self.connect_button.set_sensitive(False)
            self.advanced_button.set_sensitive(False)
            
    def update_connect_button(self, state, apbssid=None):
        """ Update the connection/disconnect button for this entry. """
        if state == misc.WIRED:
            self.disconnect_button.show()
            self.connect_button.hide()
        else:
            self.disconnect_button.hide()
            self.connect_button.show()
            
    def add_profile(self, widget):
        """ Add a profile to the profile list. """
        response = string_input("Enter a profile name", "The profile name " +
                                  "will not be used by the computer. It " +
                                  "allows you to " + 
                                  "easily distinguish between different network " +
                                  "profiles.", "Profile name:").strip()

        # if response is "" or None
        if not response:
            error(None, "Invalid profile name", block=True)
            return False

        profile_name = response
        profile_list = wired.GetWiredProfileList()
        if profile_list:
            if profile_name in profile_list:
                return False

        self.profile_help.hide()
        wired.CreateWiredNetworkProfile(profile_name, False)
        self.combo_profile_names.prepend_text(profile_name)
        self.combo_profile_names.set_active(0)
        self.advanced_dialog.prof_name = profile_name
        if self.is_full_gui:
            self.button_delete.set_sensitive(True)
            self.connect_button.set_sensitive(True)
            self.advanced_button.set_sensitive(True)

    def remove_profile(self, widget):
        """ Remove a profile from the profile list. """
        print "removing profile"
        profile_name = self.combo_profile_names.get_active_text()
        wired.DeleteWiredNetworkProfile(profile_name)
        self.combo_profile_names.remove_text(self.combo_profile_names.
                                                                 get_active())
        self.combo_profile_names.set_active(0)
        self.advanced_dialog.prof_name = self.combo_profile_names.get_active_text()
        if not wired.GetWiredProfileList():
            self.profile_help.show()
            entry = self.combo_profile_names.child
            entry.set_text("")
            if self.is_full_gui:
                self.button_delete.set_sensitive(False)
                self.advanced_button.set_sensitive(False)
                self.connect_button.set_sensitive(False)
        else:
            self.profile_help.hide()

    def toggle_default_profile(self, widget):
        """ Change the default profile. """
        if self.chkbox_default_profile.get_active():
            # Make sure there is only one default profile at a time
            wired.UnsetWiredDefault()
        wired.SetWiredProperty("default",
                               self.chkbox_default_profile.get_active())
        wired.SaveWiredNetworkProfile(self.combo_profile_names.get_active_text())

    def change_profile(self, widget):
        """ Called when a new profile is chosen from the list. """
        # Make sure the name doesn't change everytime someone types something
        if self.combo_profile_names.get_active() > -1:
            if not self.is_full_gui:
                return
            
            profile_name = self.combo_profile_names.get_active_text()
            wired.ReadWiredNetworkProfile(profile_name)

            if hasattr(self, 'advanced_dialog'):
                self.advanced_dialog.prof_name = profile_name
                self.advanced_dialog.set_values()
            
            is_default = wired.GetWiredProperty("default")
            self.chkbox_default_profile.set_active(to_bool(is_default))

    def format_entry(self, label):
        """ Help method for fetching/formatting wired properties. """
        return noneToBlankString(wired.GetWiredProperty(label))


class WirelessNetworkEntry(NetworkEntry):
    def __init__(self, networkID):
        """ Build the wireless network entry. """
        NetworkEntry.__init__(self)

        self.networkID = networkID
        self.image.set_padding(0, 0)
        self.image.set_alignment(.5, .5)
        self.image.set_size_request(60, -1)
        self.image.show()
        self.essid = noneToBlankString(wireless.GetWirelessProperty(networkID,
                                                                    "essid"))
        self.lbl_strength = GreyLabel()
        self.lbl_encryption = GreyLabel()
        self.lbl_channel = GreyLabel()
        
        print "ESSID : " + self.essid
        self.chkbox_autoconnect = gtk.CheckButton(_('Automatically connect to this network'))
        self.chkbox_neverconnect = gtk.CheckButton(_('Never connect to this network'))
        
        self.set_signal_strength(wireless.GetWirelessProperty(networkID, 
                                                              'quality'),
                                 wireless.GetWirelessProperty(networkID, 
                                                              'strength'))
        self.set_encryption(wireless.GetWirelessProperty(networkID, 
                                                         'encryption'),
                            wireless.GetWirelessProperty(networkID, 
                                                 'encryption_method')) 
        self.set_channel(wireless.GetWirelessProperty(networkID, 'channel'))
        self.name_label.set_use_markup(True)
        self.name_label.set_label("<b>%s</b>    %s    %s    %s" % (self._escape(self.essid),
                                                         self.lbl_strength.get_label(),
                                                         self.lbl_encryption.get_label(),
                                                         self.lbl_channel.get_label(),
                                                        )
                                 )
        # Add the wireless network specific parts to the NetworkEntry
        # VBox objects.
        self.vbox_top.pack_start(self.chkbox_autoconnect, False, False)
        self.vbox_top.pack_start(self.chkbox_neverconnect, False, False)

        if to_bool(self.format_entry(networkID, "automatic")):
            self.chkbox_autoconnect.set_active(True)
        else:
            self.chkbox_autoconnect.set_active(False)
        
        if to_bool(self.format_entry(networkID, "never")):
            self.chkbox_autoconnect.set_sensitive(False)
            self.connect_button.set_sensitive(False)
            self.chkbox_neverconnect.set_active(True)
        else:
            self.chkbox_neverconnect.set_active(False)

        # Connect signals.
        self.chkbox_autoconnect.connect("toggled", self.update_autoconnect)
        self.chkbox_neverconnect.connect("toggled", self.update_neverconnect)
        
        # Show everything
        self.show_all()
        self.advanced_dialog = WirelessSettingsDialog(networkID)
        self.wifides = self.connect("destroy", self.destroy_called)

    def _escape(self, val):
        """ Escapes special characters so they're displayed correctly. """
        return val.replace("&", "&amp;").replace("<", "&lt;").\
                   replace(">","&gt;").replace("'", "&apos;").replace('"', "&quot;")
    
    def save_wireless_settings(self, networkid):
        """ Save wireless network settings. """
        return self.advanced_dialog.save_settings(networkid)
    
    def update_autoconnect(self, widget=None):
        """ Called when the autoconnect checkbox is toggled. """
        wireless.SetWirelessProperty(self.networkID, "automatic",
                                     noneToString(self.chkbox_autoconnect.
                                                  get_active()))
        wireless.SaveWirelessNetworkProperty(self.networkID, "automatic")

    def update_neverconnect(self, widget=None):
        """ Called when the neverconnect checkbox is toggled. """
        wireless.SetWirelessProperty(self.networkID, "never",
                        noneToString(self.chkbox_neverconnect.get_active()))
        wireless.SaveWirelessNetworkProperty(self.networkID, "never")
        if self.chkbox_neverconnect.get_active():
            self.chkbox_autoconnect.set_sensitive(False)
            self.connect_button.set_sensitive(False)
        else:
            self.chkbox_autoconnect.set_sensitive(True)
            self.connect_button.set_sensitive(True)

    def destroy_called(self, *args):
        """ Clean up everything. """
        self.disconnect(self.wifides)
        self.advanced_dialog.destroy_called()
        del self.advanced_dialog
        super(WirelessNetworkEntry, self).destroy_called()
        self.destroy()
        del self
        
    def update_connect_button(self, state, apbssid):
        """ Update the connection/disconnect button for this entry. """
        if to_bool(self.format_entry(self.networkID, "never")):
            self.connect_button.set_sensitive(False)
        if not apbssid:
            apbssid = wireless.GetApBssid()
        if state == misc.WIRELESS and \
           apbssid == wireless.GetWirelessProperty(self.networkID, "bssid"):
            self.disconnect_button.show()
            self.connect_button.hide()
        else:
            self.disconnect_button.hide()
            self.connect_button.show()
            
    def set_signal_strength(self, strength, dbm_strength):
        """ Set the signal strength displayed in the WirelessNetworkEntry. """
        if strength:
            strength = int(strength)
        else:
            strength = -1
        if dbm_strength:
            dbm_strength = int(dbm_strength)
        else:
            dbm_strength = -100
        display_type = daemon.GetSignalDisplayType()
        if daemon.GetWPADriver() == 'ralink legacy' or display_type == 1:
            # Use the -xx dBm signal strength to display a signal icon
            # I'm not sure how accurately the dBm strength is being
            # "converted" to strength bars, so suggestions from people
            # for a better way would be welcome.
            if dbm_strength >= -60:
                signal_img = 'signal-100.png'
            elif dbm_strength >= -70:
                signal_img = 'signal-75.png'
            elif dbm_strength >= -80:
                signal_img = 'signal-50.png'
            else:
                signal_img = 'signal-25.png'
            ending = "dBm"
            disp_strength = str(dbm_strength)
        else:
            # Uses normal link quality, should be fine in most cases
            if strength > 75:
                signal_img = 'signal-100.png'
            elif strength > 50:
                signal_img = 'signal-75.png'
            elif strength > 25:
                signal_img = 'signal-50.png'
            else:
                signal_img = 'signal-25.png'
            ending = "%"
            disp_strength = str(strength)
        self.image.set_from_file(wpath.images + signal_img)
        self.lbl_strength.set_label(disp_strength + ending)
        self.image.show()
        
    def set_encryption(self, on, ttype):
        """ Set the encryption value for the WirelessNetworkEntry. """
        if on and ttype:
            self.lbl_encryption.set_label(str(ttype))
        if on and not ttype: 
            self.lbl_encryption.set_label(_('Secured'))
        if not on:
            self.lbl_encryption.set_label(_('Unsecured'))
            
    def set_channel(self, channel):
        """ Set the channel value for the WirelessNetworkEntry. """
        self.lbl_channel.set_label(_('Channel') + ' ' + str(channel))

    def format_entry(self, networkid, label):
        """ Helper method for fetching/formatting wireless properties. """
        return noneToBlankString(wireless.GetWirelessProperty(networkid, label))

        
class WirelessInformationDialog(gtk.Dialog):
    def __init__(self, networkID, parent):
        gtk.Dialog.__init__(self,parent=parent)
        
        # Make the combo box.
        self.lbl_strength = gtk.Label()
        self.lbl_strength.set_alignment(0, 0.5)
        self.lbl_encryption = gtk.Label()
        self.lbl_encryption.set_alignment(0, 0.5)
        self.lbl_mac = gtk.Label()
        self.lbl_mac.set_alignment(0, 0.5)
        self.lbl_channel = gtk.Label()
        self.lbl_channel.set_alignment(0, 0.5)
        self.lbl_mode = gtk.Label()
        self.lbl_mode.set_alignment(0, 0.5)
        self.hbox_status = gtk.HBox(False, 5)
        
        # Set the values of the network info labels.
        self.set_signal_strength(wireless.GetWirelessProperty(networkID, 
                                                              'quality'),
                                 wireless.GetWirelessProperty(networkID, 
                                                              'strength'))
        self.set_mac_address(wireless.GetWirelessProperty(networkID, 'bssid'))
        self.set_mode(wireless.GetWirelessProperty(networkID, 'mode'))
        self.set_channel(wireless.GetWirelessProperty(networkID, 'channel'))
        self.set_encryption(wireless.GetWirelessProperty(networkID,
                                                         'encryption'),
                            wireless.GetWirelessProperty(networkID, 
                                                        'encryption_method'))
        
        self.set_title('Network Information')
        vbox = self.vbox
        self.set_has_separator(False)
        table = gtk.Table(5, 2)
        table.set_col_spacings(12) 
        vbox.pack_start(table)
        
        # Pack the network status HBox.
        table.attach(LeftAlignedLabel('Signal strength:'), 0, 1, 0, 1)
        table.attach(self.lbl_strength, 1, 2, 0, 1)

        table.attach(LeftAlignedLabel('Encryption type:'), 0, 1, 1, 2)
        table.attach(self.lbl_encryption, 1, 2, 1, 2)

        table.attach(LeftAlignedLabel('Access point address:'), 0, 1, 2, 3)
        table.attach(self.lbl_mac, 1, 2, 2, 3)

        table.attach(LeftAlignedLabel('Mode:'), 0, 1, 3, 4)
        table.attach(self.lbl_mode, 1, 2, 3, 4)

        table.attach(LeftAlignedLabel('Channel:'), 0, 1, 4, 5)
        table.attach(self.lbl_channel, 1, 2, 4, 5)

        vbox.show_all()

        self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
        self.show()
        self.run()
        self.destroy()
        
    def set_signal_strength(self, strength, dbm_strength):
        """ Set the signal strength displayed in the WirelessNetworkEntry. """
        if strength is not None:
            strength = int(strength)
        else:
            strength = -1
        if dbm_strength is not None:
            dbm_strength = int(dbm_strength)
        else:
            dbm_strength = -100
        display_type = daemon.GetSignalDisplayType()
        if daemon.GetWPADriver() == 'ralink legacy' or display_type == 1:
            # Use the -xx dBm signal strength to display a signal icon
            # I'm not sure how accurately the dBm strength is being
            # "converted" to strength bars, so suggestions from people
            # for a better way would be welcome.
            if dbm_strength >= -60:
                signal_img = 'signal-100.png'
            elif dbm_strength >= -70:
                signal_img = 'signal-75.png'
            elif dbm_strength >= -80:
                signal_img = 'signal-50.png'
            else:
                signal_img = 'signal-25.png'
            ending = "dBm"
            disp_strength = str(dbm_strength)
        else:
            # Uses normal link quality, should be fine in most cases
            if strength > 75:
                signal_img = 'signal-100.png'
            elif strength > 50:
                signal_img = 'signal-75.png'
            elif strength > 25:
                signal_img = 'signal-50.png'
            else:
                signal_img = 'signal-25.png'
            ending = "%"
            disp_strength = str(strength)
        self.lbl_strength.set_label(disp_strength + ending)

    def set_mac_address(self, address):
        """ Set the MAC address for the WirelessNetworkEntry. """
        self.lbl_mac.set_label(str(address))

    def set_encryption(self, on, ttype):
        """ Set the encryption value for the WirelessNetworkEntry. """
        if on and ttype:
            self.lbl_encryption.set_label(str(ttype))
        if on and not ttype: 
            self.lbl_encryption.set_label(_('Secured'))
        if not on:
            self.lbl_encryption.set_label(_('Unsecured'))

    def set_channel(self, channel):
        """ Set the channel value for the WirelessNetworkEntry. """
        self.lbl_channel.set_label(_('Channel') + ' ' + str(channel))

    def set_mode(self, mode):
        """ Set the mode value for the WirelessNetworkEntry. """
        self.lbl_mode.set_label(str(mode))

    def format_entry(self, networkid, label):
        """ Helper method for fetching/formatting wireless properties. """
        return noneToBlankString(wireless.GetWirelessProperty(networkid, label))



""" Path configuration and functions for the wicd daemon and gui clients.

chdir() -- Change directory to the location of the current file.

"""

import os

# The path containing the wpath.py file.
current = os.path.dirname(os.path.realpath(__file__)) + '/'

# These paths can easily be modified to handle system wide installs, or
# they can be left as is if all files remain with the source directory
# layout.

# These paths are replaced when setup.py configure is run

# All directory paths *MUST* end in a /

version = '1.7.2.1'
revision = '755'
curses_revision = 'uimod'

# DIRECTORIES

lib = '/usr/lib/wicd/'
share = '/usr/share/wicd/'
etc = '/etc/wicd/'
scripts = '/etc/wicd/scripts/'
predisconnectscripts = '/etc/wicd/scripts/predisconnect'
postdisconnectscripts = '/etc/wicd/scripts/postdisconnect'
preconnectscripts = '/etc/wicd/scripts/preconnect'
postconnectscripts = '/etc/wicd/scripts/postconnect'
images = '/usr/share/pixmaps/wicd/'
encryption = '/etc/wicd/encryption/templates/'
bin = '/usr/bin/'
varlib = '/var/lib/wicd/'
networks = '/var/lib/wicd/configurations/'
log = '/var/log/wicd/'
resume = '/etc/acpi/resume.d/'
suspend = '/etc/acpi/suspend.d/'
sbin = '/usr/sbin/'
pmutils = '/usr/lib/pm-utils/sleep.d/'
dbus = '/etc/dbus-1/system.d/'
dbus_service = '/usr/share/dbus-1/system-services/'
systemd = '/lib/systemd/system/'
logrotate = '/etc/logrotate.d/'
desktop = '/usr/share/applications/'
backends= '/usr/lib/wicd/backends/'
daemon = '/usr/share/wicd/daemon/'
curses = '/usr/share/wicd/curses/'
gtk = '/usr/share/wicd/gtk/'
cli = '/usr/share/wicd/cli/'
translations = '/usr/share/locale/'
icons = '/usr/share/icons/hicolor/'
pixmaps = '/usr/share/pixmaps/'
autostart = '/etc/xdg/autostart/'
init = '/etc/rc.d/'
docdir = '/usr/doc/wicd-1.7.2.1/'
mandir = '/usr/man/'
kdedir = '/usr/share/autostart/'

# FILES

# python begins the file section
python = '/usr/bin/python'
pidfile = '/var/run/wicd/wicd.pid'
# stores something like other/wicd
# really only used in the install
initfile = 'init/slackware/rc.wicd'
# stores only the file name, i.e. wicd
initfilename = 'wicd'
wicd_group = 'netdev'
log_group = ''
log_perms = '0600'

# BOOLEANS
no_install_pmutils = False
no_install_init = False
no_install_man = False
no_install_i18n_man = False
no_install_kde = False
no_install_acpi = True
no_install_docs = False
no_install_gtk = False
no_install_ncurses = False
no_install_cli = False
no_use_notifications = False

def chdir(file):
    """Change directory to the location of the specified file.

    Keyword arguments:
    file -- the file to switch to (usually __file__)

    """
    os.chdir(os.path.dirname(os.path.realpath(file)))




#!/usr/bin/env python
# -* coding: utf-8 -*-

""" wicd-curses. (curses/urwid-based) console interface to wicd

Provides a console UI for wicd, so that people with broken X servers can
at least get a network connection.  Or those who don't like using X and/or GTK.

"""

#       Copyright (C) 2008-2009 Andrew Psaltis

#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

"""
    This contains/will contain A LOT of code from the other parts of wicd.

    This is probably due to the fact that I did not really know what I was doing
    when I started writing this.  It works, so I guess that's all that matters.

    Comments, criticisms, patches, bug reports all welcome!
"""
# Filter out a confusing urwid warning in python 2.6.
# This is valid as of urwid version 0.9.8.4
import warnings 
warnings.filterwarnings("ignore","The popen2 module is deprecated.  Use the subprocess module.") 
# UI stuff
import urwid

# DBus communication stuff
from dbus import DBusException
from dbus import version as dbus_version
# It took me a while to figure out that I have to use this.
import gobject

# Other important wicd-related stuff
from wicd import wpath
from wicd import misc
from wicd import dbusmanager

# Internal Python stuff
import sys
from time import sleep, strftime, ctime

# Curses UIs for other stuff
from curses_misc import *
from prefs_curses import PrefsDialog
import netentry_curses

from netentry_curses import WirelessSettingsDialog, WiredSettingsDialog,AdvancedSettingsDialog

from optparse import OptionParser
from os import system

# Stuff about getting the script configurer running
#from grp import getgrgid
#from os import getgroups,system

#import logging
#import logging.handler

CURSES_REV=wpath.curses_revision

# Fix strings in wicd-curses
from wicd.translations import language
from wicd.translations import _ as orig_
for i in language.keys():
    language[i] = language[i].decode('utf8')

def _(s):
    return orig_(s).decode('utf8')

########################################
##### SUPPORT CLASSES
########################################
# Yay for decorators!
def wrap_exceptions(func):
    def wrapper(*args, **kargs):
            try:
                return func(*args, **kargs)
            except KeyboardInterrupt:
                #gobject.source_remove(redraw_tag)
                loop.quit()
                ui.stop()
                print >> sys.stderr, "\n"+_('Terminated by user')
                #raise
            except DBusException:
                loop.quit()
                ui.stop()
                print >> sys.stderr,"\n"+_('DBus failure! '\
                    'This is most likely caused by the wicd daemon stopping while wicd-curses is running. '\
                    'Please restart the daemon, and then restart wicd-curses.')
                raise
            except :
                # Quit the loop
                #if 'loop' in locals():
                loop.quit()
                # Zap the screen
                ui.stop()
                # Print out standard notification:
                # This message was far too scary for humans, so it's gone now.
                # print >> sys.stderr, "\n" + _('EXCEPTION! Please report this to the maintainer and file a bug report with the backtrace below:')
                # Flush the buffer so that the notification is always above the
                # backtrace
                sys.stdout.flush()
                # Raise the exception
                raise

    wrapper.__name__ = func.__name__
    wrapper.__module__ = func.__module__
    wrapper.__dict__ = func.__dict__
    wrapper.__doc__ = func.__doc__
    return wrapper

########################################
##### SUPPORT FUNCTIONS
########################################

# Look familiar?  These two functions are clones of functions found in wicd's
# gui.py file, except that now set_status is a function passed to them.
@wrap_exceptions
def check_for_wired(wired_ip,set_status):
    """ Determine if wired is active, and if yes, set the status. """
    if wired_ip and wired.CheckPluggedIn():
        set_status(_('Connected to wired network (IP: $A)').replace('$A',wired_ip))
        return True
    else:
        return False

@wrap_exceptions
def check_for_wireless(iwconfig, wireless_ip, set_status):
    """ Determine if wireless is active, and if yes, set the status. """
    if not wireless_ip:
        return False

    network = wireless.GetCurrentNetwork(iwconfig)
    if not network:
        return False

    network = misc.to_unicode(network)
    if daemon.GetSignalDisplayType() == 0:
        strength = wireless.GetCurrentSignalStrength(iwconfig)
    else:
        strength = wireless.GetCurrentDBMStrength(iwconfig)

    if strength is None:
        return False
    strength = misc.to_unicode(daemon.FormatSignalForPrinting(strength))
    ip = misc.to_unicode(wireless_ip)
    set_status(_('Connected to $A at $B (IP: $C)').replace
                    ('$A', network).replace
                    ('$B', strength).replace
                    ('$C', ip))
    return True


# Generate the list of networks.
# Mostly borrowed/stolen from wpa_cli, since I had no clue what all of those
# DBUS interfaces do. :P
# Whatever calls this must be exception-wrapped if it is run if the UI is up
def gen_network_list():
    wiredL = wired.GetWiredProfileList()
    wlessL = []
    # This one makes a list of NetLabels
    for network_id in range(0, wireless.GetNumberOfNetworks()):
        is_active = wireless.GetCurrentSignalStrength("") != 0 and wireless.GetCurrentNetworkID(wireless.GetIwconfig())==network_id and wireless.GetWirelessIP('') != None

        label = NetLabel(network_id,is_active)
        wlessL.append(label)
    return (wiredL,wlessL)

def about_dialog(body):
    # This looks A LOT better when it is actually displayed.  I promise :-).
    # The ASCII Art "Wicd" was made from the "smslant" font on one of those
    # online ASCII big text generators.
    theText = [
('green',"   ///       \\\\\\"),"       _      ___        __\n",
('green',"  ///         \\\\\\"),"     | | /| / (_)______/ /\n",
('green'," ///           \\\\\\"),"    | |/ |/ / / __/ _  / \n",
('green',"/||  //     \\\\  ||\\"),"   |__/|__/_/\__/\_,_/  \n",
('green',"|||  ||"),"(|^|)",('green',"||  |||"),
"         ($VERSION)       \n".replace("$VERSION",daemon.Hello()),

('green',"\\||  \\\\")," |+| ",('green',"//  ||/    \n"),
('green'," \\\\\\"),"    |+|    ",('green',"///"),"      http://wicd.net\n",
('green',"  \\\\\\"),"   |+|   ",('green',"///"),"      ",_('Brought to you by:'),"\n",
('green',"   \\\\\\"),"  |+|  ",('green',"///"),"       Adam Blackburn\n",
"     ___|+|___         Dan O'Reilly\n",
"    ____|+|____        Andrew Psaltis\n",
"   |-----------|       David Paleino\n",
"-----------------------------------------------------"]
    about = TextDialog(theText,16,55,header=('header',_('About Wicd')))
    about.run(ui,body)

# Modeled after htop's help
def help_dialog(body):
    textT  = urwid.Text(('header',_('wicd-curses help')),'right') 
    textSH = urwid.Text(['This is ',('blue','wicd-curses-'+CURSES_REV),' using wicd ',unicode(daemon.Hello()),'\n'])

    textH = urwid.Text([
_('For more detailed help, consult the wicd-curses(8) man page.')+"\n",
('bold','->'),' and ',('bold','<-')," are the right and left arrows respectively.\n"])

    text1 = urwid.Text([
('bold','  H h ?'),": "+_('Display this help dialog')+"\n",
('bold','enter C'),": "+_('Connect to selected network')+"\n",
('bold','      D'),": "+_('Disconnect from all networks')+"\n",
('bold','    ESC'),": "+_('Stop a connection in progress')+"\n",
('bold','   F5 R'),": "+_('Refresh network list')+"\n",
('bold','      P'),": "+_('Preferences dialog')+"\n",
    ])
    text2 = urwid.Text([
('bold','      I'),": "+_('Scan for hidden networks')+"\n",
('bold','      S'),": "+_('Select scripts')+"\n",
('bold','      O'),": "+_('Set up Ad-hoc network')+"\n",
('bold','     ->'),": "+_('Configure selected network')+"\n",
('bold','      A'),": "+_("Display 'about' dialog")+"\n",
('bold',' F8 q Q'),": "+_('Quit wicd-curses')+"\n",
    ])
    textF = urwid.Text(_('Press any key to return.'))
    
    # textJ = urwid.Text(('important','Nobody expects the Spanish Inquisition!'))

    blank = urwid.Text('')

    cols = urwid.Columns([text1,text2])
    pile = urwid.Pile([textH,cols])
    fill = urwid.Filler(pile)
    frame = urwid.Frame(fill,header=urwid.Pile([textT,textSH]),footer=textF)
    dim = ui.get_cols_rows()
    while True:
        ui.draw_screen(dim, frame.render(dim, True))
            
        keys = ui.get_input()
        # Don't stop because someone let go of the mouse on the frame
        mouse_release = False
        for k in keys:
            if urwid.VERSION < (1, 0, 0):
                check_mouse_event = urwid.is_mouse_event
            else:
                check_mouse_event = urwid.util.is_mouse_event
            if check_mouse_event(k) and k[0] == "mouse release":
                mouse_release = True
                break
        if mouse_release :
            continue
        if 'window resize' in keys:
            dim = ui.get_cols_rows()
        elif keys:
            break

def run_configscript(parent,netname,nettype):
    configfile = wpath.etc+netname+'-settings.conf'
    if nettype != 'wired':
        header = 'profile'
    else:
        header ='BSSID'
    if nettype == 'wired':
        profname = nettype
    else:
        profname = wireless.GetWirelessProperty( int(netname),'bssid')
    theText = [
    _('To avoid various complications, wicd-curses does not support directly editing the scripts. '\
      'However, you can edit them manually. First, (as root), open the "$A" config file, and look '\
      'for the section labeled by the $B in question. In this case, this is:').
        replace('$A', configfile).replace('$B', header),
"\n\n["+profname+"]\n\n",
    _('You can also configure the wireless networks by looking for the "[<ESSID>]" field in the config file.'),
    _('Once there, you can adjust (or add) the "beforescript", "afterscript", "predisconnectscript" '\
      'and "postdisconnectscript" variables as needed, to change the preconnect, postconnect, '\
      'predisconnect and postdisconnect scripts respectively.  Note that you will be specifying '\
      'the full path to the scripts - not the actual script contents.  You will need to add/edit '\
      'the script contents separately.  Refer to the wicd manual page for more information.')
    ]
    dialog = TextDialog(theText,20,80)
    dialog.run(ui,parent)
    # This code works with many distributions, but not all of them.  So, to
    # limit complications, it has been deactivated.  If you want to run it,
    # be my guest.  Be sure to deactivate the above stuff first.
    """
    loop.quit()
    ui.stop()
    argv = netname + ' ' +nettype

    #cmd = '/usr/lib/configscript_curses.py '+argv
    cmd = wpath.lib+'configscript_curses.py '+argv
    # Check whether we can sudo.  Hopefully this is complete
    glist = []
    for i in getgroups():
        glist.append(getgrgid(i)[0])
    if 'root' in glist:
        precmd = ''
        precmdargv = ''
        postcmd = ''
    elif 'admin' in glist or 'wheel' in glist or 'sudo' in glist:
        precmd = 'sudo'
        precmdargv = ''
        postcmd = ''
    else:
        precmd = 'su'
        precmdargv = ' -c "'
        postcmd = '"'
    print "Calling command: " + precmd + precmdargv + cmd + postcmd
    sys.stdout.flush()
    system(precmd+precmdargv+cmd+postcmd)
    raw_input("Press enter!")
    main()
    """

def gen_list_header():
    if daemon.GetSignalDisplayType() == 0:
        # Allocate 25 cols for the ESSID name
        essidgap = 25
    else:
        # Need 3 more to accomodate dBm strings
        essidgap = 28
    return 'C %s %*s %9s %17s %6s %s' % ('STR ',essidgap,'ESSID','ENCRYPT','BSSID','MODE','CHNL')

########################################
##### URWID SUPPORT CLASSES
########################################

# Wireless network label
class NetLabel(urwid.WidgetWrap):
    def __init__(self, id, is_active):
        # Pick which strength measure to use based on what the daemon says
        # gap allocates more space to the first module
        if daemon.GetSignalDisplayType() == 0:
            strenstr = 'quality'
            gap = 4 # Allow for 100%
        else:
            strenstr = 'strength'
            gap = 7 # -XX dbm = 7
        self.id = id
        # All of that network property stuff
        self.stren = daemon.FormatSignalForPrinting(
                str(wireless.GetWirelessProperty(id, strenstr)))
        self.essid = wireless.GetWirelessProperty(id, 'essid')
        self.bssid = wireless.GetWirelessProperty(id, 'bssid')

        if wireless.GetWirelessProperty(id, 'encryption'):
            self.encrypt = wireless.GetWirelessProperty(id,'encryption_method')
        else:
            self.encrypt = _('Unsecured')

        self.mode  = wireless.GetWirelessProperty(id, 'mode') # Master, Ad-Hoc
        self.channel = wireless.GetWirelessProperty(id, 'channel')
        theString = '  %-*s %25s %9s %17s %6s %4s' % (gap,
                self.stren,self.essid,self.encrypt,self.bssid,self.mode,self.channel)
        if is_active:
            theString = '>'+theString[1:]
            w = urwid.AttrWrap(SelText(theString),'connected','connected focus')
        else:
            w = urwid.AttrWrap(SelText(theString),'body','focus')

        self.__super.__init__(w)
    def selectable(self):
        return True
    def keypress(self,size,key):
        return self._w.keypress(size,key)
    def connect(self):
        wireless.ConnectWireless(self.id)
        
class WiredComboBox(ComboBox):
    """
    list : the list of wired network profiles.  The rest is self-explanitory.
    """
    def __init__(self,list):
        self.ADD_PROFILE = '---'+_('Add a new profile')+'---'
        self.__super.__init__(use_enter=False)
        self.set_list(list)

    def set_list(self,list):
        self.theList = list
        id=0
        wiredL=[]
        is_active = wireless.GetWirelessIP('') == None and wired.GetWiredIP('') != None
        for profile in list:
            theString = '%4s   %25s' % (id, profile)
            # Tag if no wireless IP present, and wired one is
            if is_active:
                theString = '>'+theString[1:]
                
            wiredL.append(theString)
            id+=1
        wiredL.append(self.ADD_PROFILE)
        if is_active:
            self.attrs = ('connected','editnfc')
            self.focus_attr = 'connected focus'
        else :
            self.attrs = ('body','editnfc')
            self.focus_attr = 'focus'
        self.list = wiredL
        if self.theList != []:
            wired.ReadWiredNetworkProfile(self.get_selected_profile())

    def keypress(self,size,key):
        prev_focus = self.get_focus()[1]
        key = ComboBox.keypress(self,size,key)
        if key == ' ':
            if self.get_focus()[1] == len(self.list)-1:
                dialog = InputDialog(('header',_('Add a new wired profile')),7,30)
                exitcode,name = dialog.run(ui,self.parent)
                if exitcode == 0:
                    name = name.strip()
                    if not name:
                        error(ui,self.parent,'Invalid profile name')
                        self.set_focus(prev_focus)
                        return key

                    wired.CreateWiredNetworkProfile(name,False)
                    self.set_list(wired.GetWiredProfileList())
                    self.rebuild_combobox()
                self.set_focus(prev_focus)
            else:
                wired.ReadWiredNetworkProfile(self.get_selected_profile())
        if key == 'delete':
            if len(self.theList) == 1:
                error(self.ui,self.parent,_('wicd-curses does not support deleting the last wired profile.  Try renaming it ("F2")'))
                return key
            wired.DeleteWiredNetworkProfile(self.get_selected_profile())
            # Return to the top of the list if something is deleted.

            if wired.GetDefaultWiredNetwork() != None:
                self.set_focus(self.theList.index(wired.GetDefaultWiredNetwork()))
            else:
                prev_focus -= 1
                self.set_focus(prev_focus)
            self.set_list(wired.GetWiredProfileList())
            self.rebuild_combobox()
        if key == 'f2':
            dialog = InputDialog(('header',_('Rename wired profile')),7,30,
                    edit_text=unicode(self.get_selected_profile()))
            exitcode,name = dialog.run(ui,self.parent)
            if exitcode == 0:
                # Save the new one, then kill the old one
                wired.SaveWiredNetworkProfile(name)
                wired.DeleteWiredNetworkProfile(self.get_selected_profile())
                self.set_list(wired.GetWiredProfileList())
                self.set_focus(self.theList.index(name))
                self.rebuild_combobox()
        return key

    def get_selected_profile(self):
        """Get the selected wired profile"""
        loc = self.get_focus()[1]
        return self.theList[loc]

# Dialog2 that initiates an Ad-Hoc network connection
class AdHocDialog(Dialog2):
    def __init__(self):
        essid_t = _('ESSID')
        ip_t = _('IP')
        channel_t = _('Channel')
        key_t = "    " + _('Key')
        use_ics_t =  _('Activate Internet Connection Sharing')
        use_encrypt_t = _('Use Encryption (WEP only)')

        self.essid_edit = DynEdit(essid_t)
        self.ip_edit = DynEdit(ip_t)
        self.channel_edit = DynIntEdit(channel_t)
        self.key_edit = DynEdit(key_t,sensitive=False)

        self.use_ics_chkb = urwid.CheckBox(use_ics_t)
        self.use_encrypt_chkb = urwid.CheckBox(use_encrypt_t,
                on_state_change=self.encrypt_callback)

        blank = urwid.Text('')

        # Set defaults
        self.essid_edit.set_edit_text("My_Adhoc_Network")
        self.ip_edit.set_edit_text("169.254.12.10")
        self.channel_edit.set_edit_text("3")

        l = [self.essid_edit,self.ip_edit,self.channel_edit,blank,
                self.use_ics_chkb,self.use_encrypt_chkb,self.key_edit]
        body = urwid.ListBox(l)

        header = ('header', _('Create an Ad-Hoc Network'))
        Dialog2.__init__(self, header, 15, 50, body)
        self.add_buttons([(_('OK'),1),(_('Cancel'),-1)])
        self.frame.set_focus('body')

    def encrypt_callback(self,chkbox,new_state,user_info=None):
        self.key_edit.set_sensitive(new_state)

    def unhandled_key(self, size, k):
        if k in ('up','page up'):
            self.frame.set_focus('body')
        if k in ('down','page down'):
            self.frame.set_focus('footer')
        if k == 'enter':
            # pass enter to the "ok" button
            self.frame.set_focus('footer')
            self.buttons.set_focus(0)
            self.view.keypress( size, k )
    def on_exit(self,exitcode):
        data = ( self.essid_edit.get_edit_text(),
                 self.ip_edit.get_edit_text().strip(),
                 self.channel_edit.get_edit_text(),
                 self.use_ics_chkb.get_state(),
                 self.use_encrypt_chkb.get_state(),
                 self.key_edit.get_edit_text())
        return exitcode, data

########################################
##### APPLICATION INTERFACE CLASS
########################################
# The Whole Shebang
class appGUI():
    """The UI itself, all glory belongs to it!"""
    def __init__(self):
        global loop
        self.size = ui.get_cols_rows()
        # Happy screen saying that you can't do anything because we're scanning
        # for networks.  :-)
        self.screen_locker = urwid.Filler(urwid.Text(('important',_('Scanning networks... stand by...')), align='center'))
        self.no_wlan = urwid.Filler(urwid.Text(('important',_('No wireless networks found.')), align='center'))
        self.TITLE = _('Wicd Curses Interface')
        self.WIRED_IDX = 1
        self.WLESS_IDX = 3

        header = urwid.AttrWrap(urwid.Text(self.TITLE,align='right'), 'header')
        self.wiredH=urwid.Filler(urwid.Text(_('Wired Networks')))
        self.list_header=urwid.AttrWrap(urwid.Text(gen_list_header()),'listbar')
        self.wlessH=NSelListBox([urwid.Text(_('Wireless Networks')),self.list_header])

        # Init this earlier to make update_status happy
        self.update_tag = None

        # FIXME: This should be two variables
        self.focusloc = [1,0]

        # These are empty to make sure that things go my way.
        wiredL,wlessL = [],[]

        self.frame = None
        self.diag = None

        self.wiredCB = urwid.Filler(WiredComboBox(wiredL))
        self.wlessLB = urwid.ListBox(wlessL)
        self.update_netlist(force_check=True,firstrun=True)
        
        # Keymappings proposed by nanotube in #wicd
        keys = [
                ('H' ,_('Help'),None),
                ('right',_('Config'),None),
                #('  ','         ',None),
                ('K' , _('RfKill'),None),
                ('C' ,_('Connect'),None),
                ('D' ,_('Disconn'),None),
                ('R' ,_('Refresh'),None),
                ('P' ,_('Prefs'),None),
                ('I' ,_('Hidden'),None),
                ('A' ,_('About'),None),
                ('Q' ,_('Quit'),loop.quit)
               ]

        self.primaryCols = OptCols(keys,self.handle_keys)
        self.status_label = urwid.AttrWrap(urwid.Text(''),'important')
        self.footer2 = urwid.Columns([self.status_label])
        self.footerList = urwid.Pile([self.primaryCols,self.footer2])

        self.frame = urwid.Frame(self.thePile,
                                 header=header,
                                 footer=self.footerList)
        self.wiredCB.get_body().build_combobox(self.frame,ui,3)

        # Init the other columns used in the program
        self.init_other_optcols()

        self.frame.set_body(self.thePile)
        # Booleans gallore!
        self.prev_state    = False
        self.connecting    = False
        self.screen_locked = False
        self.do_diag_lock = False #Whether the screen is locked beneath a dialog
        self.diag_type = 'none' # The type of dialog that is up
        self.scanning = False

        self.pref = None

        self.update_status()

        #self.max_wait = ui.max_wait

    def doScan(self, sync=False):
        self.scanning = True
        wireless.Scan(False)

    def init_other_optcols(self):
        # The "tabbed" preferences dialog
        self.prefCols = OptCols( [ ('f10',_('OK')),
                                   ('page up',_('Tab Left'),),
                                   ('page down', _('Tab Right')),
                                   ('esc',_('Cancel')) ], self.handle_keys)
        self.confCols = OptCols( [ ('f10',_('OK')),
                                   ('esc',_('Cancel')) ],self.handle_keys)

    # Does what it says it does
    def lock_screen(self):
        if self.diag_type == 'pref':
            self.do_diag_lock = True
            return True
        self.frame.set_body(self.screen_locker)
        self.screen_locked = True
        self.update_ui()

    def unlock_screen(self):
        if self.do_diag_lock:
            self.do_diag_lock = False
            return True
        self.update_netlist(force_check=True)
        if not self.diag:
            self.frame.set_body(self.thePile)
        self.screen_locked = False
        self.update_ui()

    def raise_hidden_network_dialog(self):
        dialog = InputDialog(('header',_('Select Hidden Network ESSID')),7,30,_('Scan'))
        exitcode,hidden = dialog.run(ui,self.frame)
        if exitcode != -1:
            # That dialog will sit there for a while if I don't get rid of it
            self.update_ui()
            wireless.SetHiddenNetworkESSID(misc.noneToString(hidden))
            wireless.Scan(False)
        wireless.SetHiddenNetworkESSID("")
        
    def update_focusloc(self):
        # Location of last known focus is remapped to current location.
        # This might need to be cleaned up later.

        if self.thePile.get_focus() == self.wiredCB: 
            wlessorwired = self.WIRED_IDX
            where = self.thePile.get_focus().get_body().get_focus()[1]
        else: #self.thePile.get_focus() == self.wlessLB :
            wlessorwired = self.WLESS_IDX
            if self.wlessLB == self.no_wlan:
                where = None
            else: 
                where = self.thePile.get_focus().get_focus()[1]
                #where = self.wlessLB.get_focus()[1]
        self.focusloc = [wlessorwired,where]
    
    # Be clunky until I get to a later stage of development.
    # Update the list of networks.  Usually called by DBus.
    @wrap_exceptions
    def update_netlist(self,state=None, x=None, force_check=False,firstrun=False):
        # Don't even try to do this if we are running a dialog
        if self.diag:
            return
        # Run focus-collecting code if we are not running this for the first
        # time
        if not firstrun:
            self.update_focusloc()
            self.list_header.set_text(gen_list_header())
        """ Updates the overall network list."""
        if not state:
            state, x = daemon.GetConnectionStatus()
        if force_check or self.prev_state != state:
            wiredL,wlessL = gen_network_list()

            self.wiredCB.get_body().set_list(wiredL)
            self.wiredCB.get_body().build_combobox(self.frame,ui,3)
            if len(wlessL) != 0:
                if self.wlessLB == self.no_wlan:
                    self.wlessLB = urwid.ListBox(wlessL)
                else:
                    self.wlessLB.body = urwid.SimpleListWalker(wlessL)
            else:
                self.wlessLB = self.no_wlan
            if daemon.GetAlwaysShowWiredInterface() or wired.CheckPluggedIn():
                self.thePile = urwid.Pile([('fixed',1,self.wiredH),
                                           ('fixed',1,self.wiredCB),
                                           ('fixed',2,self.wlessH),
                                                      self.wlessLB] )
                if not firstrun:
                    self.frame.body = self.thePile

                self.thePile.set_focus(self.focusloc[0])
                if self.focusloc[0] == self.WIRED_IDX:
                    self.thePile.get_focus().get_body().set_focus(self.focusloc[1])
                else:
                    if self.wlessLB != self.no_wlan:
                        self.thePile.get_focus().set_focus(self.focusloc[1])
                    else:
                        self.thePile.set_focus(self.wiredCB)
            else:
                self.thePile = urwid.Pile([('fixed',2,self.wlessH),self.wlessLB] )
                if not firstrun:
                    self.frame.body = self.thePile
                if self.focusloc[1] == None:
                    self.focusloc[1] = 0
                if self.wlessLB != self.no_wlan:
                    self.wlessLB.set_focus(self.focusloc[1])

        self.prev_state = state
        if not firstrun:
            self.update_ui()
        if firstrun:
            if wired.GetDefaultWiredNetwork() != None:
                self.wiredCB.get_body().set_focus(wired.GetWiredProfileList().index(wired.GetDefaultWiredNetwork()))

    # Update the footer/status bar
    conn_status = False
    @wrap_exceptions
    def update_status(self):
        wired_connecting = wired.CheckIfWiredConnecting()
        wireless_connecting = wireless.CheckIfWirelessConnecting()
        self.connecting = wired_connecting or wireless_connecting
        
        fast = not daemon.NeedsExternalCalls()
        if self.connecting: 
            if not self.conn_status:
                self.conn_status = True
                gobject.timeout_add(250,self.set_connecting_status,fast)
            return True
        else:
            if check_for_wired(wired.GetWiredIP(''),self.set_status):
                return True
            if not fast:
                iwconfig = wireless.GetIwconfig()
            else:
                iwconfig = ''
            if check_for_wireless(iwconfig, wireless.GetWirelessIP(""),
                    self.set_status):
                return True
            else:
                self.set_status(_('Not connected'))
                self.update_ui()
                return True

    def set_connecting_status(self,fast):
        wired_connecting = wired.CheckIfWiredConnecting()
        wireless_connecting = wireless.CheckIfWirelessConnecting()
        if wireless_connecting:
            if not fast:
                iwconfig = wireless.GetIwconfig()
            else:
                iwconfig = ''
            essid = wireless.GetCurrentNetwork(iwconfig)
            stat = wireless.CheckWirelessConnectingMessage()
            return self.set_status("%s: %s" % (essid, stat), True)
        if wired_connecting:
            return self.set_status(_('Wired Network') +
                    ': ' + wired.CheckWiredConnectingMessage(), True)
        else:
            self.conn_status=False
            return False

    # Cheap little indicator stating that we are actually connecting
    twirl = ['|','/','-','\\']
    tcount = 0 # Counter for said indicator
    def set_status(self,text,from_idle=False):
        # Set the status text, usually called by the update_status method
        # from_idle : a check to see if we are being called directly from the
        # mainloop
        # If we are being called as the result of trying to connect to
        # something, and we aren't connecting to something, return False
        # immediately.
        if from_idle and not self.connecting:
            self.update_status()
            self.conn_status=False
            return False
        toAppend = ''
        # If we are connecting and being called from the idle function, spin
        # the wheel.
        if from_idle and self.connecting:
            # This is probably the wrong way to do this, but it works for now.
            self.tcount+=1
            toAppend=self.twirl[self.tcount % 4]
        self.status_label.set_text(text+' '+toAppend)
        self.update_ui()
        return True

    def dbus_scan_finished(self):
        # I'm pretty sure that I'll need this later.
        #if not self.connecting:
        #    gobject.idle_add(self.refresh_networks, None, False, None)
        self.unlock_screen()
        self.scanning = False

    def dbus_scan_started(self):
        self.scanning = True
        if self.diag_type == 'conf':
            self.restore_primary()
        self.lock_screen()

    def restore_primary(self):
        self.diag_type = 'none'
        if self.do_diag_lock or self.scanning:
            self.frame.set_body(self.screen_locker)
            self.do_diag_lock = False
        else:
            self.frame.set_body(self.thePile)
        self.diag = None
        self.frame.set_footer(urwid.Pile([self.primaryCols,self.footer2]))
        self.update_ui()

    def handle_keys(self,keys):
        if not self.diag:
            # Handle keystrokes
            if "f8" in keys or 'Q' in keys or 'q' in keys:
                loop.quit()
                #return False
            if "f5" in keys or 'R' in keys:
                self.lock_screen()
                self.doScan()
            if 'k' in keys or 'K' in keys:
                wireless.SwitchRfKill()
                self.update_netlist()
            if "D" in keys:
                # Disconnect from all networks.
                daemon.Disconnect()
                self.update_netlist()
            if 'right' in keys:
                if not self.scanning:
                    focus = self.thePile.get_focus()
                    self.frame.set_footer(urwid.Pile([self.confCols,self.footer2]))
                    if focus == self.wiredCB:
                        self.diag = WiredSettingsDialog(self.wiredCB.get_body().get_selected_profile(),self.frame)
                        self.diag.ready_widgets(ui,self.frame)
                        self.frame.set_body(self.diag)
                    else:
                        # wireless list only other option
                        wid,pos  = self.thePile.get_focus().get_focus()
                        self.diag = WirelessSettingsDialog(pos,self.frame)
                        self.diag.ready_widgets(ui,self.frame)
                        self.frame.set_body(self.diag)
                    self.diag_type = 'conf'
            if "enter" in keys or 'C' in keys:
                if not self.scanning:
                    focus = self.frame.body.get_focus()
                    if focus == self.wiredCB:
                        self.special = focus
                        self.connect("wired",0)
                    else:
                        # wless list only other option, if it is around
                        if self.wlessLB != self.no_wlan:
                            wid,pos = self.thePile.get_focus().get_focus()
                            self.connect("wireless",pos)
            if "esc" in keys:
                # Force disconnect here if connection in progress
                if self.connecting:
                    daemon.CancelConnect()
                    # Prevents automatic reconnecting if that option is enabled
                    daemon.SetForcedDisconnect(True)
            if "P" in keys:
                if not self.pref:
                    self.pref = PrefsDialog(self.frame,(0,1),ui,
                            dbusmanager.get_dbus_ifaces()) 
                self.pref.load_settings()
                self.pref.ready_widgets(ui,self.frame)
                self.frame.set_footer(urwid.Pile([self.prefCols,self.footer2]))
                self.diag = self.pref
                self.diag_type = 'pref'
                self.frame.set_body(self.diag)
                # Halt here, keypress gets passed to the dialog otherwise
                return True
            if "A" in keys:
                about_dialog(self.frame)
            if "I" in keys:
                self.raise_hidden_network_dialog()
            if "H" in keys or 'h' in keys or '?' in keys:
                # FIXME I shouldn't need this, OptCols messes up this one
                # particular button
                if not self.diag:
                    help_dialog(self.frame)
            if "S" in keys:
                focus = self.thePile.get_focus()
                if focus == self.wiredCB:
                    nettype = 'wired'
                    netname = self.wiredCB.get_body().get_selected_profile()
                else:
                    nettype = 'wireless'
                    netname = str(self.wlessLB.get_focus()[1])
                run_configscript(self.frame,netname,nettype)
            if "O" in keys:
                exitcode,data = AdHocDialog().run(ui,self.frame)
                #data = (essid,ip,channel,use_ics,use_encrypt,key_edit)
                if exitcode == 1:
                    wireless.CreateAdHocNetwork(data[0],
                                                data[2],
                                                data[1], "WEP",
                                                data[5],
                                                data[4], False)
            
        for k in keys:
            if urwid.VERSION < (1, 0, 0):
                check_mouse_event = urwid.is_mouse_event
            else:
                check_mouse_event = urwid.util.is_mouse_event
            if check_mouse_event(k):
                event, button, col, row = k
                self.frame.mouse_event( self.size,
                        event, button, col, row,
                        focus=True)
                continue
            k = self.frame.keypress(self.size,k)
            if self.diag:
                if  k == 'esc' or k == 'q' or k == 'Q':
                    self.restore_primary()
                    break
                if k == 'f10':
                    self.diag.save_settings()
                    self.restore_primary()
                    break
            if k == "window resize":
                self.size = ui.get_cols_rows()
                continue

    def call_update_ui(self,source,cb_condition):           
        self.update_ui(True)                                
        return True                                         
                
    # Redraw the screen
    @wrap_exceptions
    def update_ui(self,from_key=False):
        if not ui._started:
            return False

        input_data = ui.get_input_nonblocking()
        # Resolve any "alarms" in the waiting
        self.handle_keys(input_data[1])

        # Update the screen
        canvas = self.frame.render( (self.size),True )
        ui.draw_screen((self.size),canvas)
        # Get the input data
        if self.update_tag != None:
            gobject.source_remove(self.update_tag)
        #if from_key:
        return False

    def connect(self, nettype, networkid, networkentry=None):
        """ Initiates the connection process in the daemon. """
        if nettype == "wireless":
            wireless.ConnectWireless(networkid)
        elif nettype == "wired":
            wired.ConnectWired()
        self.update_status()

########################################
##### INITIALIZATION FUNCTIONS
########################################

def main():
    global ui, dlogger
    # We are not python.
    misc.RenameProcess('wicd-curses')

    import urwid.raw_display
    ui = urwid.raw_display.Screen()

    #if options.debug:
    #    dlogger = logging.getLogger("Debug")
    #    dlogger.setLevel(logging.DEBUG)
    #    dlogger.debug("wicd-curses debug logging started")

    # Default Color scheme.
    # Other potential color schemes can be found at:
    # http://excess.org/urwid/wiki/RecommendedPalette

    # Thanks to nanotube on #wicd for helping with this
    ui.register_palette([
        ('body','default','default'),
        ('focus','black','light gray'),
        ('header','light blue','default'),
        ('important','light red','default'),
        ('connected','dark green','default'),
        ('connected focus','black','dark green'),
        ('editcp', 'default', 'default', 'standout'),
        ('editbx', 'light gray', 'dark blue'),
        ('editfc', 'white','dark blue', 'bold'),
        ('editnfc','brown','default','bold'),
        ('tab active','dark green','light gray'),
        ('infobar','light gray','dark blue'),
        ('listbar','light blue','default'),
        # Simple colors around text
        ('green','dark green','default'),
        ('blue','light blue','default'),
        ('red','dark red','default'),
        ('bold','white','black','bold')])
    # This is a wrapper around a function that calls another a function that
    # is a wrapper around a infinite loop.  Fun.
    urwid.set_encoding('utf8')
    ui.run_wrapper(run)

@wrap_exceptions
def run():
    global loop
    loop = gobject.MainLoop()
    
    ui.set_mouse_tracking()
    app = appGUI()

    # Connect signals and whatnot to UI screen control functions
    bus.add_signal_receiver(app.dbus_scan_finished, 'SendEndScanSignal',
                            'org.wicd.daemon.wireless')
    bus.add_signal_receiver(app.dbus_scan_started, 'SendStartScanSignal',
                            'org.wicd.daemon.wireless')
    # I've left this commented out many times.
    bus.add_signal_receiver(app.update_netlist, 'StatusChanged',
                            'org.wicd.daemon')
    # Update the connection status on the bottom every 2 s.
    gobject.timeout_add(2000,app.update_status)

    # Get input file descriptors and add callbacks to the ui-updating function
    fds = ui.get_input_descriptors()
    for fd in fds:
        gobject.io_add_watch(fd, gobject.IO_IN,app.call_update_ui)
    app.update_ui()
    loop.run()

# Mostly borrowed from gui.py
def setup_dbus(force=True):
    global bus, daemon, wireless, wired
    try:
        dbusmanager.connect_to_dbus()
    except DBusException:
        print >> sys.stderr, _("Can't connect to the daemon, trying to start it automatically...")
    bus = dbusmanager.get_bus()
    dbus_ifaces = dbusmanager.get_dbus_ifaces()
    daemon = dbus_ifaces['daemon']
    wireless = dbus_ifaces['wireless']
    wired = dbus_ifaces['wired']

    if not daemon:
        print 'Error connecting to wicd via D-Bus.  Please make sure the wicd service is running.'
        sys.exit(3)

    netentry_curses.dbus_init(dbus_ifaces)
    return True

setup_dbus()

########################################
##### MAIN ENTRY POINT
########################################
if __name__ == '__main__':
    try:
        parser = OptionParser(version="wicd-curses-%s (using wicd %s)" % (CURSES_REV,daemon.Hello()), prog="wicd-curses")
    except Exception, e:
        if "DBus.Error.AccessDenied" in e.get_dbus_name():
            print _('ERROR: wicd-curses was denied access to the wicd daemon: '\
                'please check that your user is in the "$A" group.').\
                replace('$A','\033[1;34m' + wpath.wicd_group + '\033[0m')
            sys.exit(1)
        else:
            raise
    #parser.add_option("-d", "--debug",action="store_true"
    #        ,dest='debug',help="enable logging of wicd-curses (currently does nothing)")

    (options,args) = parser.parse_args()
    main()



#!/usr/bin/env python

#
#    Copyright (C) 1999-2006  Keith Dart <keith@kdart.com>
#    Copyright (C) 2008-2009  Dan O'Reilly <oreilldf@gmail.com>
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.

"""

Managing logfile rotation. A ManagedLog object is a file-like object that
rotates itself when a maximum size is reached.

"""

import sys
import os
import time

class SizeError(IOError):
    pass

class LogFile(file):
    """LogFile(name, [mode="w"], [maxsize=360000])
    
    Opens a new file object. After writing <maxsize> bytes a SizeError
    will be raised.
    
    """
    def __init__(self, name, mode="a", maxsize=360000):
        super(LogFile, self).__init__(name, mode)
        self.maxsize = maxsize
        self.eol = True
        try:
            self.written = os.fstat(self.fileno())[6]
        except OSError:
            self.written = 0

    def write(self, data):
        self.written += len(data)
        
        data = data.decode('utf-8').encode('utf-8')
        if len(data) <= 0: return
        if self.eol:
            super(LogFile, self).write(self.get_time() + ' :: ')
            self.eol = False

        if data[-1] == '\n':
            self.eol = True
            data = data[:-1]

        super(LogFile, self).write(data.replace(
                                    '\n', '\n' + self.get_time() + ' :: '))
        if self.eol:
            super(LogFile, self).write('\n')
            
        self.flush()
        if self.written > self.maxsize:
            raise SizeError
        
    def get_time(self):
        """ Return a string with the current time nicely formatted.

        The format of the returned string is yyyy/mm/dd HH:MM:SS

        """
        x = time.localtime()
        return ''.join([
            str(x[0]).rjust(4, '0'), '/', str(x[1]).rjust(2, '0'), '/',
            str(x[2]).rjust(2, '0'), ' ', str(x[3]).rjust(2, '0'), ':',
            str(x[4]).rjust(2, '0'), ':', str(x[5]).rjust(2, '0')])

    def rotate(self):
        return rotate(self)

    def note(self, text):
        """Writes a specially formated note text to the file.
        
        The note starts with the string '\\n#*=' so you can easily filter them.
        
        """
        self.write("\n#*===== %s =====\n" % (text,))


class ManagedLog(object):
    """ManagedLog(name, [maxsize=360000], [maxsave=9])
    
    A ManagedLog instance is a persistent log object. Write data with the
    write() method. The log size and rotation is handled automatically.

    """
    def __init__(self, name, maxsize=360000, maxsave=3):
        if not os.path.exists(os.path.dirname(name)):
            os.makedirs(os.path.dirname(name))
        self._lf = LogFile(name, "a", maxsize)
        self.maxsave = maxsave

    def __repr__(self):
        return "%s(%r, %r, %r)" % (self.__class__.__name__, self._lf.name, 
                                   self._lf.maxsize, self.maxsave)

    def write(self, data):
        try:
            self._lf.write(data)
        except SizeError:
            self._lf = rotate(self._lf, self.maxsave)

    def note(self, data):
        try:
            self._lf.note(data)
        except SizeError:
            self._lf = rotate(self._lf, self.maxsave)

    def written(self):
        return self._lf.written

    def rotate(self): 
        self._lf = rotate(self._lf, self.maxsave)

    # auto-delegate remaining methods (but you should not read or seek an open
    # log file).
    def __getattr__(self, name):
        return getattr(self._lf, name)


# useful for logged stdout for daemon processes
class ManagedStdio(ManagedLog):
    def write(self, data):
        try:
            self._lf.write(data)
        except SizeError:
            sys.stdout.flush()
            sys.stderr.flush()
            self._lf = rotate(self._lf, self.maxsave)
            fd = self._lf.fileno()
            os.dup2(fd, 1)
            os.dup2(fd, 2)
            sys.stdout = sys.stderr = self


def rotate(fileobj, maxsave=9):
    name = fileobj.name
    mode = fileobj.mode
    maxsize = fileobj.maxsize
    fileobj.close()
    shiftlogs(name, maxsave)
    return LogFile(name, mode, maxsize)


# assumes basename logfile is closed.
def shiftlogs(basename, maxsave):
    topname = "%s.%d" % (basename, maxsave)
    if os.path.isfile(topname):
        os.unlink(topname)

    for i in range(maxsave, 0, -1):
        oldname = "%s.%d" % (basename, i)
        newname = "%s.%d" % (basename, i+1)
        try:
            os.rename(oldname, newname)
        except OSError:
            pass
    try:
        os.rename(basename, "%s.1" % (basename))
    except OSError:
        pass


def open(name, maxsize=360000, maxsave=9):
    return ManagedLog(name, maxsize, maxsave)

def writelog(logobj, data):
    try:
        logobj.write(data)
    except SizeError:
        return rotate(logobj)
    else:
        return logobj
